# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "network vpn-connection wait",
)
class Wait(AAZWaitCommand):
    """Place the CLI in a waiting state until a condition is met.
    """

    _aaz_info = {
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.network/connections/{}", "2024-07-01"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.name = AAZStrArg(
            options=["-n", "--name"],
            help="Connection name.",
            required=True,
            id_part="name",
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.VirtualNetworkGatewayConnectionsGet(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=False)
        return result

    class VirtualNetworkGatewayConnectionsGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/connections/{virtualNetworkGatewayConnectionName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
                **self.serialize_url_param(
                    "virtualNetworkGatewayConnectionName", self.ctx.args.name,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2024-07-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.etag = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.id = AAZStrType()
            _schema_on_200.location = AAZStrType()
            _schema_on_200.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.properties = AAZObjectType(
                flags={"required": True, "client_flatten": True},
            )
            _schema_on_200.tags = AAZDictType()
            _schema_on_200.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.properties
            properties.authorization_key = AAZStrType(
                serialized_name="authorizationKey",
            )
            properties.connection_mode = AAZStrType(
                serialized_name="connectionMode",
            )
            properties.connection_protocol = AAZStrType(
                serialized_name="connectionProtocol",
            )
            properties.connection_status = AAZStrType(
                serialized_name="connectionStatus",
                flags={"read_only": True},
            )
            properties.connection_type = AAZStrType(
                serialized_name="connectionType",
                flags={"required": True},
            )
            properties.dpd_timeout_seconds = AAZIntType(
                serialized_name="dpdTimeoutSeconds",
            )
            properties.egress_bytes_transferred = AAZIntType(
                serialized_name="egressBytesTransferred",
                flags={"read_only": True},
            )
            properties.egress_nat_rules = AAZListType(
                serialized_name="egressNatRules",
            )
            properties.enable_bgp = AAZBoolType(
                serialized_name="enableBgp",
            )
            properties.enable_private_link_fast_path = AAZBoolType(
                serialized_name="enablePrivateLinkFastPath",
            )
            properties.express_route_gateway_bypass = AAZBoolType(
                serialized_name="expressRouteGatewayBypass",
            )
            properties.gateway_custom_bgp_ip_addresses = AAZListType(
                serialized_name="gatewayCustomBgpIpAddresses",
            )
            properties.ingress_bytes_transferred = AAZIntType(
                serialized_name="ingressBytesTransferred",
                flags={"read_only": True},
            )
            properties.ingress_nat_rules = AAZListType(
                serialized_name="ingressNatRules",
            )
            properties.ipsec_policies = AAZListType(
                serialized_name="ipsecPolicies",
            )
            properties.local_network_gateway2 = AAZObjectType(
                serialized_name="localNetworkGateway2",
            )
            properties.peer = AAZObjectType()
            _WaitHelper._build_schema_sub_resource_read(properties.peer)
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.resource_guid = AAZStrType(
                serialized_name="resourceGuid",
                flags={"read_only": True},
            )
            properties.routing_weight = AAZIntType(
                serialized_name="routingWeight",
            )
            properties.shared_key = AAZStrType(
                serialized_name="sharedKey",
            )
            properties.traffic_selector_policies = AAZListType(
                serialized_name="trafficSelectorPolicies",
            )
            properties.tunnel_connection_status = AAZListType(
                serialized_name="tunnelConnectionStatus",
                flags={"read_only": True},
            )
            properties.tunnel_properties = AAZListType(
                serialized_name="tunnelProperties",
            )
            properties.use_local_azure_ip_address = AAZBoolType(
                serialized_name="useLocalAzureIpAddress",
            )
            properties.use_policy_based_traffic_selectors = AAZBoolType(
                serialized_name="usePolicyBasedTrafficSelectors",
            )
            properties.virtual_network_gateway1 = AAZObjectType(
                serialized_name="virtualNetworkGateway1",
                flags={"required": True},
            )
            _WaitHelper._build_schema_virtual_network_gateway_read(properties.virtual_network_gateway1)
            properties.virtual_network_gateway2 = AAZObjectType(
                serialized_name="virtualNetworkGateway2",
            )
            _WaitHelper._build_schema_virtual_network_gateway_read(properties.virtual_network_gateway2)

            egress_nat_rules = cls._schema_on_200.properties.egress_nat_rules
            egress_nat_rules.Element = AAZObjectType()
            _WaitHelper._build_schema_sub_resource_read(egress_nat_rules.Element)

            gateway_custom_bgp_ip_addresses = cls._schema_on_200.properties.gateway_custom_bgp_ip_addresses
            gateway_custom_bgp_ip_addresses.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.gateway_custom_bgp_ip_addresses.Element
            _element.custom_bgp_ip_address = AAZStrType(
                serialized_name="customBgpIpAddress",
                flags={"required": True},
            )
            _element.ip_configuration_id = AAZStrType(
                serialized_name="ipConfigurationId",
                flags={"required": True},
            )

            ingress_nat_rules = cls._schema_on_200.properties.ingress_nat_rules
            ingress_nat_rules.Element = AAZObjectType()
            _WaitHelper._build_schema_sub_resource_read(ingress_nat_rules.Element)

            ipsec_policies = cls._schema_on_200.properties.ipsec_policies
            ipsec_policies.Element = AAZObjectType()
            _WaitHelper._build_schema_ipsec_policy_read(ipsec_policies.Element)

            local_network_gateway2 = cls._schema_on_200.properties.local_network_gateway2
            local_network_gateway2.etag = AAZStrType(
                flags={"read_only": True},
            )
            local_network_gateway2.id = AAZStrType()
            local_network_gateway2.location = AAZStrType()
            local_network_gateway2.name = AAZStrType(
                flags={"read_only": True},
            )
            local_network_gateway2.properties = AAZObjectType(
                flags={"required": True, "client_flatten": True},
            )
            local_network_gateway2.tags = AAZDictType()
            local_network_gateway2.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.properties.local_network_gateway2.properties
            properties.bgp_settings = AAZObjectType(
                serialized_name="bgpSettings",
            )
            _WaitHelper._build_schema_bgp_settings_read(properties.bgp_settings)
            properties.fqdn = AAZStrType()
            properties.gateway_ip_address = AAZStrType(
                serialized_name="gatewayIpAddress",
            )
            properties.local_network_address_space = AAZObjectType(
                serialized_name="localNetworkAddressSpace",
            )
            _WaitHelper._build_schema_address_space_read(properties.local_network_address_space)
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.resource_guid = AAZStrType(
                serialized_name="resourceGuid",
                flags={"read_only": True},
            )

            tags = cls._schema_on_200.properties.local_network_gateway2.tags
            tags.Element = AAZStrType()

            traffic_selector_policies = cls._schema_on_200.properties.traffic_selector_policies
            traffic_selector_policies.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.traffic_selector_policies.Element
            _element.local_address_ranges = AAZListType(
                serialized_name="localAddressRanges",
                flags={"required": True},
            )
            _element.remote_address_ranges = AAZListType(
                serialized_name="remoteAddressRanges",
                flags={"required": True},
            )

            local_address_ranges = cls._schema_on_200.properties.traffic_selector_policies.Element.local_address_ranges
            local_address_ranges.Element = AAZStrType()

            remote_address_ranges = cls._schema_on_200.properties.traffic_selector_policies.Element.remote_address_ranges
            remote_address_ranges.Element = AAZStrType()

            tunnel_connection_status = cls._schema_on_200.properties.tunnel_connection_status
            tunnel_connection_status.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.tunnel_connection_status.Element
            _element.connection_status = AAZStrType(
                serialized_name="connectionStatus",
                flags={"read_only": True},
            )
            _element.egress_bytes_transferred = AAZIntType(
                serialized_name="egressBytesTransferred",
                flags={"read_only": True},
            )
            _element.ingress_bytes_transferred = AAZIntType(
                serialized_name="ingressBytesTransferred",
                flags={"read_only": True},
            )
            _element.last_connection_established_utc_time = AAZStrType(
                serialized_name="lastConnectionEstablishedUtcTime",
                flags={"read_only": True},
            )
            _element.tunnel = AAZStrType(
                flags={"read_only": True},
            )

            tunnel_properties = cls._schema_on_200.properties.tunnel_properties
            tunnel_properties.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.tunnel_properties.Element
            _element.bgp_peering_address = AAZStrType(
                serialized_name="bgpPeeringAddress",
            )
            _element.tunnel_ip_address = AAZStrType(
                serialized_name="tunnelIpAddress",
            )

            tags = cls._schema_on_200.tags
            tags.Element = AAZStrType()

            return cls._schema_on_200


class _WaitHelper:
    """Helper class for Wait"""

    _schema_address_space_read = None

    @classmethod
    def _build_schema_address_space_read(cls, _schema):
        if cls._schema_address_space_read is not None:
            _schema.address_prefixes = cls._schema_address_space_read.address_prefixes
            return

        cls._schema_address_space_read = _schema_address_space_read = AAZObjectType()

        address_space_read = _schema_address_space_read
        address_space_read.address_prefixes = AAZListType(
            serialized_name="addressPrefixes",
        )

        address_prefixes = _schema_address_space_read.address_prefixes
        address_prefixes.Element = AAZStrType()

        _schema.address_prefixes = cls._schema_address_space_read.address_prefixes

    _schema_bgp_settings_read = None

    @classmethod
    def _build_schema_bgp_settings_read(cls, _schema):
        if cls._schema_bgp_settings_read is not None:
            _schema.asn = cls._schema_bgp_settings_read.asn
            _schema.bgp_peering_address = cls._schema_bgp_settings_read.bgp_peering_address
            _schema.bgp_peering_addresses = cls._schema_bgp_settings_read.bgp_peering_addresses
            _schema.peer_weight = cls._schema_bgp_settings_read.peer_weight
            return

        cls._schema_bgp_settings_read = _schema_bgp_settings_read = AAZObjectType()

        bgp_settings_read = _schema_bgp_settings_read
        bgp_settings_read.asn = AAZIntType()
        bgp_settings_read.bgp_peering_address = AAZStrType(
            serialized_name="bgpPeeringAddress",
        )
        bgp_settings_read.bgp_peering_addresses = AAZListType(
            serialized_name="bgpPeeringAddresses",
        )
        bgp_settings_read.peer_weight = AAZIntType(
            serialized_name="peerWeight",
        )

        bgp_peering_addresses = _schema_bgp_settings_read.bgp_peering_addresses
        bgp_peering_addresses.Element = AAZObjectType()

        _element = _schema_bgp_settings_read.bgp_peering_addresses.Element
        _element.custom_bgp_ip_addresses = AAZListType(
            serialized_name="customBgpIpAddresses",
        )
        _element.default_bgp_ip_addresses = AAZListType(
            serialized_name="defaultBgpIpAddresses",
            flags={"read_only": True},
        )
        _element.ipconfiguration_id = AAZStrType(
            serialized_name="ipconfigurationId",
        )
        _element.tunnel_ip_addresses = AAZListType(
            serialized_name="tunnelIpAddresses",
            flags={"read_only": True},
        )

        custom_bgp_ip_addresses = _schema_bgp_settings_read.bgp_peering_addresses.Element.custom_bgp_ip_addresses
        custom_bgp_ip_addresses.Element = AAZStrType()

        default_bgp_ip_addresses = _schema_bgp_settings_read.bgp_peering_addresses.Element.default_bgp_ip_addresses
        default_bgp_ip_addresses.Element = AAZStrType()

        tunnel_ip_addresses = _schema_bgp_settings_read.bgp_peering_addresses.Element.tunnel_ip_addresses
        tunnel_ip_addresses.Element = AAZStrType()

        _schema.asn = cls._schema_bgp_settings_read.asn
        _schema.bgp_peering_address = cls._schema_bgp_settings_read.bgp_peering_address
        _schema.bgp_peering_addresses = cls._schema_bgp_settings_read.bgp_peering_addresses
        _schema.peer_weight = cls._schema_bgp_settings_read.peer_weight

    _schema_ipsec_policy_read = None

    @classmethod
    def _build_schema_ipsec_policy_read(cls, _schema):
        if cls._schema_ipsec_policy_read is not None:
            _schema.dh_group = cls._schema_ipsec_policy_read.dh_group
            _schema.ike_encryption = cls._schema_ipsec_policy_read.ike_encryption
            _schema.ike_integrity = cls._schema_ipsec_policy_read.ike_integrity
            _schema.ipsec_encryption = cls._schema_ipsec_policy_read.ipsec_encryption
            _schema.ipsec_integrity = cls._schema_ipsec_policy_read.ipsec_integrity
            _schema.pfs_group = cls._schema_ipsec_policy_read.pfs_group
            _schema.sa_data_size_kilobytes = cls._schema_ipsec_policy_read.sa_data_size_kilobytes
            _schema.sa_life_time_seconds = cls._schema_ipsec_policy_read.sa_life_time_seconds
            return

        cls._schema_ipsec_policy_read = _schema_ipsec_policy_read = AAZObjectType()

        ipsec_policy_read = _schema_ipsec_policy_read
        ipsec_policy_read.dh_group = AAZStrType(
            serialized_name="dhGroup",
            flags={"required": True},
        )
        ipsec_policy_read.ike_encryption = AAZStrType(
            serialized_name="ikeEncryption",
            flags={"required": True},
        )
        ipsec_policy_read.ike_integrity = AAZStrType(
            serialized_name="ikeIntegrity",
            flags={"required": True},
        )
        ipsec_policy_read.ipsec_encryption = AAZStrType(
            serialized_name="ipsecEncryption",
            flags={"required": True},
        )
        ipsec_policy_read.ipsec_integrity = AAZStrType(
            serialized_name="ipsecIntegrity",
            flags={"required": True},
        )
        ipsec_policy_read.pfs_group = AAZStrType(
            serialized_name="pfsGroup",
            flags={"required": True},
        )
        ipsec_policy_read.sa_data_size_kilobytes = AAZIntType(
            serialized_name="saDataSizeKilobytes",
            flags={"required": True},
        )
        ipsec_policy_read.sa_life_time_seconds = AAZIntType(
            serialized_name="saLifeTimeSeconds",
            flags={"required": True},
        )

        _schema.dh_group = cls._schema_ipsec_policy_read.dh_group
        _schema.ike_encryption = cls._schema_ipsec_policy_read.ike_encryption
        _schema.ike_integrity = cls._schema_ipsec_policy_read.ike_integrity
        _schema.ipsec_encryption = cls._schema_ipsec_policy_read.ipsec_encryption
        _schema.ipsec_integrity = cls._schema_ipsec_policy_read.ipsec_integrity
        _schema.pfs_group = cls._schema_ipsec_policy_read.pfs_group
        _schema.sa_data_size_kilobytes = cls._schema_ipsec_policy_read.sa_data_size_kilobytes
        _schema.sa_life_time_seconds = cls._schema_ipsec_policy_read.sa_life_time_seconds

    _schema_sub_resource_read = None

    @classmethod
    def _build_schema_sub_resource_read(cls, _schema):
        if cls._schema_sub_resource_read is not None:
            _schema.id = cls._schema_sub_resource_read.id
            return

        cls._schema_sub_resource_read = _schema_sub_resource_read = AAZObjectType()

        sub_resource_read = _schema_sub_resource_read
        sub_resource_read.id = AAZStrType()

        _schema.id = cls._schema_sub_resource_read.id

    _schema_virtual_network_gateway_read = None

    @classmethod
    def _build_schema_virtual_network_gateway_read(cls, _schema):
        if cls._schema_virtual_network_gateway_read is not None:
            _schema.etag = cls._schema_virtual_network_gateway_read.etag
            _schema.extended_location = cls._schema_virtual_network_gateway_read.extended_location
            _schema.id = cls._schema_virtual_network_gateway_read.id
            _schema.identity = cls._schema_virtual_network_gateway_read.identity
            _schema.location = cls._schema_virtual_network_gateway_read.location
            _schema.name = cls._schema_virtual_network_gateway_read.name
            _schema.properties = cls._schema_virtual_network_gateway_read.properties
            _schema.tags = cls._schema_virtual_network_gateway_read.tags
            _schema.type = cls._schema_virtual_network_gateway_read.type
            return

        cls._schema_virtual_network_gateway_read = _schema_virtual_network_gateway_read = AAZObjectType()

        virtual_network_gateway_read = _schema_virtual_network_gateway_read
        virtual_network_gateway_read.etag = AAZStrType(
            flags={"read_only": True},
        )
        virtual_network_gateway_read.extended_location = AAZObjectType(
            serialized_name="extendedLocation",
        )
        virtual_network_gateway_read.id = AAZStrType()
        virtual_network_gateway_read.identity = AAZIdentityObjectType()
        virtual_network_gateway_read.location = AAZStrType()
        virtual_network_gateway_read.name = AAZStrType(
            flags={"read_only": True},
        )
        virtual_network_gateway_read.properties = AAZObjectType(
            flags={"required": True, "client_flatten": True},
        )
        virtual_network_gateway_read.tags = AAZDictType()
        virtual_network_gateway_read.type = AAZStrType(
            flags={"read_only": True},
        )

        extended_location = _schema_virtual_network_gateway_read.extended_location
        extended_location.name = AAZStrType()
        extended_location.type = AAZStrType()

        identity = _schema_virtual_network_gateway_read.identity
        identity.principal_id = AAZStrType(
            serialized_name="principalId",
            flags={"read_only": True},
        )
        identity.tenant_id = AAZStrType(
            serialized_name="tenantId",
            flags={"read_only": True},
        )
        identity.type = AAZStrType()
        identity.user_assigned_identities = AAZDictType(
            serialized_name="userAssignedIdentities",
        )

        user_assigned_identities = _schema_virtual_network_gateway_read.identity.user_assigned_identities
        user_assigned_identities.Element = AAZObjectType()

        _element = _schema_virtual_network_gateway_read.identity.user_assigned_identities.Element
        _element.client_id = AAZStrType(
            serialized_name="clientId",
            flags={"read_only": True},
        )
        _element.principal_id = AAZStrType(
            serialized_name="principalId",
            flags={"read_only": True},
        )

        properties = _schema_virtual_network_gateway_read.properties
        properties.active_active = AAZBoolType(
            serialized_name="activeActive",
        )
        properties.admin_state = AAZStrType(
            serialized_name="adminState",
        )
        properties.allow_remote_vnet_traffic = AAZBoolType(
            serialized_name="allowRemoteVnetTraffic",
        )
        properties.allow_virtual_wan_traffic = AAZBoolType(
            serialized_name="allowVirtualWanTraffic",
        )
        properties.auto_scale_configuration = AAZObjectType(
            serialized_name="autoScaleConfiguration",
        )
        properties.bgp_settings = AAZObjectType(
            serialized_name="bgpSettings",
        )
        cls._build_schema_bgp_settings_read(properties.bgp_settings)
        properties.custom_routes = AAZObjectType(
            serialized_name="customRoutes",
        )
        cls._build_schema_address_space_read(properties.custom_routes)
        properties.disable_ip_sec_replay_protection = AAZBoolType(
            serialized_name="disableIPSecReplayProtection",
        )
        properties.enable_bgp = AAZBoolType(
            serialized_name="enableBgp",
        )
        properties.enable_bgp_route_translation_for_nat = AAZBoolType(
            serialized_name="enableBgpRouteTranslationForNat",
        )
        properties.enable_dns_forwarding = AAZBoolType(
            serialized_name="enableDnsForwarding",
        )
        properties.enable_high_bandwidth_vpn_gateway = AAZBoolType(
            serialized_name="enableHighBandwidthVpnGateway",
        )
        properties.enable_private_ip_address = AAZBoolType(
            serialized_name="enablePrivateIpAddress",
        )
        properties.gateway_default_site = AAZObjectType(
            serialized_name="gatewayDefaultSite",
        )
        cls._build_schema_sub_resource_read(properties.gateway_default_site)
        properties.gateway_type = AAZStrType(
            serialized_name="gatewayType",
        )
        properties.inbound_dns_forwarding_endpoint = AAZStrType(
            serialized_name="inboundDnsForwardingEndpoint",
            flags={"read_only": True},
        )
        properties.ip_configurations = AAZListType(
            serialized_name="ipConfigurations",
        )
        properties.nat_rules = AAZListType(
            serialized_name="natRules",
        )
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.resiliency_model = AAZStrType(
            serialized_name="resiliencyModel",
        )
        properties.resource_guid = AAZStrType(
            serialized_name="resourceGuid",
            flags={"read_only": True},
        )
        properties.sku = AAZObjectType()
        properties.v_net_extended_location_resource_id = AAZStrType(
            serialized_name="vNetExtendedLocationResourceId",
        )
        properties.virtual_network_gateway_migration_status = AAZObjectType(
            serialized_name="virtualNetworkGatewayMigrationStatus",
        )
        properties.virtual_network_gateway_policy_groups = AAZListType(
            serialized_name="virtualNetworkGatewayPolicyGroups",
        )
        properties.vpn_client_configuration = AAZObjectType(
            serialized_name="vpnClientConfiguration",
        )
        properties.vpn_gateway_generation = AAZStrType(
            serialized_name="vpnGatewayGeneration",
        )
        properties.vpn_type = AAZStrType(
            serialized_name="vpnType",
        )

        auto_scale_configuration = _schema_virtual_network_gateway_read.properties.auto_scale_configuration
        auto_scale_configuration.bounds = AAZObjectType()

        bounds = _schema_virtual_network_gateway_read.properties.auto_scale_configuration.bounds
        bounds.max = AAZIntType()
        bounds.min = AAZIntType()

        ip_configurations = _schema_virtual_network_gateway_read.properties.ip_configurations
        ip_configurations.Element = AAZObjectType()

        _element = _schema_virtual_network_gateway_read.properties.ip_configurations.Element
        _element.etag = AAZStrType(
            flags={"read_only": True},
        )
        _element.id = AAZStrType()
        _element.name = AAZStrType()
        _element.properties = AAZObjectType(
            flags={"client_flatten": True},
        )

        properties = _schema_virtual_network_gateway_read.properties.ip_configurations.Element.properties
        properties.private_ip_address = AAZStrType(
            serialized_name="privateIPAddress",
            flags={"read_only": True},
        )
        properties.private_ip_allocation_method = AAZStrType(
            serialized_name="privateIPAllocationMethod",
        )
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.public_ip_address = AAZObjectType(
            serialized_name="publicIPAddress",
        )
        cls._build_schema_sub_resource_read(properties.public_ip_address)
        properties.subnet = AAZObjectType()
        cls._build_schema_sub_resource_read(properties.subnet)

        nat_rules = _schema_virtual_network_gateway_read.properties.nat_rules
        nat_rules.Element = AAZObjectType()

        _element = _schema_virtual_network_gateway_read.properties.nat_rules.Element
        _element.etag = AAZStrType(
            flags={"read_only": True},
        )
        _element.id = AAZStrType()
        _element.name = AAZStrType()
        _element.properties = AAZObjectType(
            flags={"client_flatten": True},
        )
        _element.type = AAZStrType(
            flags={"read_only": True},
        )

        properties = _schema_virtual_network_gateway_read.properties.nat_rules.Element.properties
        properties.external_mappings = AAZListType(
            serialized_name="externalMappings",
        )
        properties.internal_mappings = AAZListType(
            serialized_name="internalMappings",
        )
        properties.ip_configuration_id = AAZStrType(
            serialized_name="ipConfigurationId",
        )
        properties.mode = AAZStrType()
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.type = AAZStrType()

        external_mappings = _schema_virtual_network_gateway_read.properties.nat_rules.Element.properties.external_mappings
        external_mappings.Element = AAZObjectType()
        cls._build_schema_vpn_nat_rule_mapping_read(external_mappings.Element)

        internal_mappings = _schema_virtual_network_gateway_read.properties.nat_rules.Element.properties.internal_mappings
        internal_mappings.Element = AAZObjectType()
        cls._build_schema_vpn_nat_rule_mapping_read(internal_mappings.Element)

        sku = _schema_virtual_network_gateway_read.properties.sku
        sku.capacity = AAZIntType(
            flags={"read_only": True},
        )
        sku.name = AAZStrType()
        sku.tier = AAZStrType()

        virtual_network_gateway_migration_status = _schema_virtual_network_gateway_read.properties.virtual_network_gateway_migration_status
        virtual_network_gateway_migration_status.error_message = AAZStrType(
            serialized_name="errorMessage",
        )
        virtual_network_gateway_migration_status.phase = AAZStrType()
        virtual_network_gateway_migration_status.state = AAZStrType()

        virtual_network_gateway_policy_groups = _schema_virtual_network_gateway_read.properties.virtual_network_gateway_policy_groups
        virtual_network_gateway_policy_groups.Element = AAZObjectType()

        _element = _schema_virtual_network_gateway_read.properties.virtual_network_gateway_policy_groups.Element
        _element.etag = AAZStrType(
            flags={"read_only": True},
        )
        _element.id = AAZStrType()
        _element.name = AAZStrType()
        _element.properties = AAZObjectType(
            flags={"client_flatten": True},
        )

        properties = _schema_virtual_network_gateway_read.properties.virtual_network_gateway_policy_groups.Element.properties
        properties.is_default = AAZBoolType(
            serialized_name="isDefault",
            flags={"required": True},
        )
        properties.policy_members = AAZListType(
            serialized_name="policyMembers",
            flags={"required": True},
        )
        properties.priority = AAZIntType(
            flags={"required": True},
        )
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.vng_client_connection_configurations = AAZListType(
            serialized_name="vngClientConnectionConfigurations",
            flags={"read_only": True},
        )

        policy_members = _schema_virtual_network_gateway_read.properties.virtual_network_gateway_policy_groups.Element.properties.policy_members
        policy_members.Element = AAZObjectType()

        _element = _schema_virtual_network_gateway_read.properties.virtual_network_gateway_policy_groups.Element.properties.policy_members.Element
        _element.attribute_type = AAZStrType(
            serialized_name="attributeType",
        )
        _element.attribute_value = AAZStrType(
            serialized_name="attributeValue",
        )
        _element.name = AAZStrType()

        vng_client_connection_configurations = _schema_virtual_network_gateway_read.properties.virtual_network_gateway_policy_groups.Element.properties.vng_client_connection_configurations
        vng_client_connection_configurations.Element = AAZObjectType()
        cls._build_schema_sub_resource_read(vng_client_connection_configurations.Element)

        vpn_client_configuration = _schema_virtual_network_gateway_read.properties.vpn_client_configuration
        vpn_client_configuration.aad_audience = AAZStrType(
            serialized_name="aadAudience",
        )
        vpn_client_configuration.aad_issuer = AAZStrType(
            serialized_name="aadIssuer",
        )
        vpn_client_configuration.aad_tenant = AAZStrType(
            serialized_name="aadTenant",
        )
        vpn_client_configuration.radius_server_address = AAZStrType(
            serialized_name="radiusServerAddress",
        )
        vpn_client_configuration.radius_server_secret = AAZStrType(
            serialized_name="radiusServerSecret",
        )
        vpn_client_configuration.radius_servers = AAZListType(
            serialized_name="radiusServers",
        )
        vpn_client_configuration.vng_client_connection_configurations = AAZListType(
            serialized_name="vngClientConnectionConfigurations",
        )
        vpn_client_configuration.vpn_authentication_types = AAZListType(
            serialized_name="vpnAuthenticationTypes",
        )
        vpn_client_configuration.vpn_client_address_pool = AAZObjectType(
            serialized_name="vpnClientAddressPool",
        )
        cls._build_schema_address_space_read(vpn_client_configuration.vpn_client_address_pool)
        vpn_client_configuration.vpn_client_ipsec_policies = AAZListType(
            serialized_name="vpnClientIpsecPolicies",
        )
        vpn_client_configuration.vpn_client_protocols = AAZListType(
            serialized_name="vpnClientProtocols",
        )
        vpn_client_configuration.vpn_client_revoked_certificates = AAZListType(
            serialized_name="vpnClientRevokedCertificates",
        )
        vpn_client_configuration.vpn_client_root_certificates = AAZListType(
            serialized_name="vpnClientRootCertificates",
        )

        radius_servers = _schema_virtual_network_gateway_read.properties.vpn_client_configuration.radius_servers
        radius_servers.Element = AAZObjectType()

        _element = _schema_virtual_network_gateway_read.properties.vpn_client_configuration.radius_servers.Element
        _element.radius_server_address = AAZStrType(
            serialized_name="radiusServerAddress",
            flags={"required": True},
        )
        _element.radius_server_score = AAZIntType(
            serialized_name="radiusServerScore",
        )
        _element.radius_server_secret = AAZStrType(
            serialized_name="radiusServerSecret",
        )

        vng_client_connection_configurations = _schema_virtual_network_gateway_read.properties.vpn_client_configuration.vng_client_connection_configurations
        vng_client_connection_configurations.Element = AAZObjectType()

        _element = _schema_virtual_network_gateway_read.properties.vpn_client_configuration.vng_client_connection_configurations.Element
        _element.etag = AAZStrType(
            flags={"read_only": True},
        )
        _element.id = AAZStrType()
        _element.name = AAZStrType()
        _element.properties = AAZObjectType(
            flags={"client_flatten": True},
        )

        properties = _schema_virtual_network_gateway_read.properties.vpn_client_configuration.vng_client_connection_configurations.Element.properties
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.virtual_network_gateway_policy_groups = AAZListType(
            serialized_name="virtualNetworkGatewayPolicyGroups",
            flags={"required": True},
        )
        properties.vpn_client_address_pool = AAZObjectType(
            serialized_name="vpnClientAddressPool",
            flags={"required": True},
        )
        cls._build_schema_address_space_read(properties.vpn_client_address_pool)

        virtual_network_gateway_policy_groups = _schema_virtual_network_gateway_read.properties.vpn_client_configuration.vng_client_connection_configurations.Element.properties.virtual_network_gateway_policy_groups
        virtual_network_gateway_policy_groups.Element = AAZObjectType()
        cls._build_schema_sub_resource_read(virtual_network_gateway_policy_groups.Element)

        vpn_authentication_types = _schema_virtual_network_gateway_read.properties.vpn_client_configuration.vpn_authentication_types
        vpn_authentication_types.Element = AAZStrType()

        vpn_client_ipsec_policies = _schema_virtual_network_gateway_read.properties.vpn_client_configuration.vpn_client_ipsec_policies
        vpn_client_ipsec_policies.Element = AAZObjectType()
        cls._build_schema_ipsec_policy_read(vpn_client_ipsec_policies.Element)

        vpn_client_protocols = _schema_virtual_network_gateway_read.properties.vpn_client_configuration.vpn_client_protocols
        vpn_client_protocols.Element = AAZStrType()

        vpn_client_revoked_certificates = _schema_virtual_network_gateway_read.properties.vpn_client_configuration.vpn_client_revoked_certificates
        vpn_client_revoked_certificates.Element = AAZObjectType()

        _element = _schema_virtual_network_gateway_read.properties.vpn_client_configuration.vpn_client_revoked_certificates.Element
        _element.etag = AAZStrType(
            flags={"read_only": True},
        )
        _element.id = AAZStrType()
        _element.name = AAZStrType()
        _element.properties = AAZObjectType(
            flags={"client_flatten": True},
        )

        properties = _schema_virtual_network_gateway_read.properties.vpn_client_configuration.vpn_client_revoked_certificates.Element.properties
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.thumbprint = AAZStrType()

        vpn_client_root_certificates = _schema_virtual_network_gateway_read.properties.vpn_client_configuration.vpn_client_root_certificates
        vpn_client_root_certificates.Element = AAZObjectType()

        _element = _schema_virtual_network_gateway_read.properties.vpn_client_configuration.vpn_client_root_certificates.Element
        _element.etag = AAZStrType(
            flags={"read_only": True},
        )
        _element.id = AAZStrType()
        _element.name = AAZStrType()
        _element.properties = AAZObjectType(
            flags={"required": True, "client_flatten": True},
        )

        properties = _schema_virtual_network_gateway_read.properties.vpn_client_configuration.vpn_client_root_certificates.Element.properties
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.public_cert_data = AAZStrType(
            serialized_name="publicCertData",
            flags={"required": True},
        )

        tags = _schema_virtual_network_gateway_read.tags
        tags.Element = AAZStrType()

        _schema.etag = cls._schema_virtual_network_gateway_read.etag
        _schema.extended_location = cls._schema_virtual_network_gateway_read.extended_location
        _schema.id = cls._schema_virtual_network_gateway_read.id
        _schema.identity = cls._schema_virtual_network_gateway_read.identity
        _schema.location = cls._schema_virtual_network_gateway_read.location
        _schema.name = cls._schema_virtual_network_gateway_read.name
        _schema.properties = cls._schema_virtual_network_gateway_read.properties
        _schema.tags = cls._schema_virtual_network_gateway_read.tags
        _schema.type = cls._schema_virtual_network_gateway_read.type

    _schema_vpn_nat_rule_mapping_read = None

    @classmethod
    def _build_schema_vpn_nat_rule_mapping_read(cls, _schema):
        if cls._schema_vpn_nat_rule_mapping_read is not None:
            _schema.address_space = cls._schema_vpn_nat_rule_mapping_read.address_space
            _schema.port_range = cls._schema_vpn_nat_rule_mapping_read.port_range
            return

        cls._schema_vpn_nat_rule_mapping_read = _schema_vpn_nat_rule_mapping_read = AAZObjectType()

        vpn_nat_rule_mapping_read = _schema_vpn_nat_rule_mapping_read
        vpn_nat_rule_mapping_read.address_space = AAZStrType(
            serialized_name="addressSpace",
        )
        vpn_nat_rule_mapping_read.port_range = AAZStrType(
            serialized_name="portRange",
        )

        _schema.address_space = cls._schema_vpn_nat_rule_mapping_read.address_space
        _schema.port_range = cls._schema_vpn_nat_rule_mapping_read.port_range


__all__ = ["Wait"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import abc

# pylint: disable=protected-access, too-few-public-methods


class AAZBaseValue:
    """ Base field value class. Value should combined with a value type definition. """

    def __init__(self, schema, data):
        self._schema = schema   # value type definition
        if isinstance(data, AAZValuePatch):
            self._data = data.data
            self._is_patch = True
        else:
            self._data = data
            self._is_patch = False

    @abc.abstractmethod
    def to_serialized_data(self, processor=None, **kwargs):
        """ serialize value into python build-in type """
        raise NotImplementedError()


class AAZBaseType:
    """ Base field type class. """

    _ValueCls = None
    _PatchDataCls = None

    def __init__(self, options=None, nullable=False, serialized_name=None, flags=None):
        assert issubclass(self._ValueCls, AAZBaseValue)
        self._serialized_name = serialized_name  # name used in serialization
        self._options = options
        self._name = None
        self._nullable = nullable   # when true, specifies that `None` is a valid value
        # support special properties such as `client_flatten` or `read_only`
        self._flags = {} if flags is None else flags

    @abc.abstractmethod
    def process_data(self, data, **kwargs):
        """Process data, make sure the return data followed the Type definition."""
        raise NotImplementedError()


class _AAZUndefinedType:
    """Internal class for AAZUndefined global const"""

    def __str__(self):
        return 'Undefined'

    def __repr__(self):
        return 'Undefined'

    def __eq__(self, other):
        return self is other

    def __ne__(self, other):
        return self is not other

    def __bool__(self):
        return False

    def __copy__(self):
        return self

    def __deepcopy__(self, *args, **kwargs):
        return self

    def __lt__(self, other):
        self._cmp_err(other, '<')

    def __gt__(self, other):
        self._cmp_err(other, '>')

    def __le__(self, other):
        self._cmp_err(other, '<=')

    def __ge__(self, other):
        self._cmp_err(other, '>=')

    def _cmp_err(self, other, op):
        raise TypeError(f"unorderable types: {self.__class__.__name__}() {op} {other.__class__.__name__}()")


# AAZ framework defines a global const called AAZUndefined. Which is used to show a field is not defined.
# In order to different with `None` value
# This value is used in aaz package only.
AAZUndefined = _AAZUndefinedType()


class AAZValuePatch:
    """Value patch is used to support partial value definition."""

    @classmethod
    def build(cls, schema):
        """build a value patch from value type"""
        if schema._PatchDataCls:
            return cls(data=schema._PatchDataCls())
        return cls(data=AAZUndefined)

    def __init__(self, data):
        self.data = data


class _AAZBlankArgValueType:
    """Internal class for AAZUndefined global const"""

    def __str__(self):
        return 'BlankArgValue'

    def __repr__(self):
        return 'BlankArgValue'

    def __eq__(self, other):
        return self is other

    def __ne__(self, other):
        return self is not other

    def __bool__(self):
        return False

    def __copy__(self):
        return self

    def __deepcopy__(self, *args, **kwargs):
        return self

    def __lt__(self, other):
        self._cmp_err(other, '<')

    def __gt__(self, other):
        self._cmp_err(other, '>')

    def __le__(self, other):
        self._cmp_err(other, '<=')

    def __ge__(self, other):
        self._cmp_err(other, '>=')

    def _cmp_err(self, other, op):
        raise TypeError(f"unorderable types: {self.__class__.__name__}() {op} {other.__class__.__name__}()")


# AAZ framework defines a global const called AAZUndefined. Which is used to show a field is not defined.
# In order to different with `None` value
# This value is used in aaz package only.
AAZBlankArgValue = _AAZBlankArgValueType()


def has_value(arg_value):
    if isinstance(arg_value, AAZBaseValue):
        # handle patch value for list, object, dict
        return arg_value.to_serialized_data() != AAZUndefined
    return arg_value != AAZUndefined



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "storage share-rm show",
)
class Show(AAZCommand):
    """Show the properties for a specified Azure file share or share snapshot.

    :example: Show the properties for an Azure file share 'myfileshare' under the storage account 'mystorageaccount' (account name) in resource group 'MyResourceGroup'.
        az storage share-rm show -g MyResourceGroup --storage-account mystorageaccount --name myfileshare

    :example: Show the properties for an Azure file share 'myfileshare' under the storage account 'mystorageaccount' (account id).
        az storage share-rm show --storage-account mystorageaccount --name myfileshare

    :example: Show the properties of an Azure file share by resource id.
        az storage share-rm show --ids file-share-id

    :example: Show the properties of an Azure file share snapshot
        az storage share-rm show --ids file-share-id --snapshot "2021-03-25T05:29:56.0000000Z"
    """

    _aaz_info = {
        "version": "2024-01-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.storage/storageaccounts/{}/fileservices/default/shares/{}", "2024-01-01"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.snapshot = AAZStrArg(
            options=["--snapshot"],
            help="Optional, used to delete a snapshot.The DateTime value that specifies the share snapshot to retrieve.",
        )
        _args_schema.storage_account = AAZStrArg(
            options=["--account-name", "--storage-account"],
            help="The name of the storage account within the specified resource group. Storage account names must be between 3 and 24 characters in length and use numbers and lower-case letters only.",
            required=True,
            id_part="name",
            fmt=AAZStrArgFormat(
                pattern="^[a-z0-9]+$",
                max_length=24,
                min_length=3,
            ),
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.share_name = AAZStrArg(
            options=["-n", "--name", "--share-name"],
            help="The name of the file share within the specified storage account. File share names must be between 3 and 63 characters in length and use numbers, lower-case letters and dash (-) only. Every dash (-) character must be immediately preceded and followed by a letter or number.",
            required=True,
            id_part="child_name_2",
            fmt=AAZStrArgFormat(
                max_length=63,
                min_length=3,
            ),
        )
        _args_schema.expand = AAZStrArg(
            options=["--expand"],
            help="Optional, used to expand the properties within share's properties. Valid values are: deleted, snapshots. Should be passed as a string with delimiter ','",
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.FileSharesGet(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class FileSharesGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Storage/storageAccounts/{accountName}/fileServices/default/shares/{shareName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "accountName", self.ctx.args.storage_account,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "shareName", self.ctx.args.share_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "$expand", self.ctx.args.expand,
                ),
                **self.serialize_query_param(
                    "api-version", "2024-01-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "x-ms-snapshot", self.ctx.args.snapshot,
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.etag = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_200.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.properties
            properties.access_tier = AAZStrType(
                serialized_name="accessTier",
            )
            properties.access_tier_change_time = AAZStrType(
                serialized_name="accessTierChangeTime",
                flags={"read_only": True},
            )
            properties.access_tier_status = AAZStrType(
                serialized_name="accessTierStatus",
                flags={"read_only": True},
            )
            properties.deleted = AAZBoolType(
                flags={"read_only": True},
            )
            properties.deleted_time = AAZStrType(
                serialized_name="deletedTime",
                flags={"read_only": True},
            )
            properties.enabled_protocols = AAZStrType(
                serialized_name="enabledProtocols",
            )
            properties.file_share_paid_bursting = AAZObjectType(
                serialized_name="fileSharePaidBursting",
            )
            properties.included_burst_iops = AAZIntType(
                serialized_name="includedBurstIops",
                flags={"read_only": True},
            )
            properties.last_modified_time = AAZStrType(
                serialized_name="lastModifiedTime",
                flags={"read_only": True},
            )
            properties.lease_duration = AAZStrType(
                serialized_name="leaseDuration",
                flags={"read_only": True},
            )
            properties.lease_state = AAZStrType(
                serialized_name="leaseState",
                flags={"read_only": True},
            )
            properties.lease_status = AAZStrType(
                serialized_name="leaseStatus",
                flags={"read_only": True},
            )
            properties.max_burst_credits_for_iops = AAZIntType(
                serialized_name="maxBurstCreditsForIops",
                flags={"read_only": True},
            )
            properties.metadata = AAZDictType()
            properties.next_allowed_provisioned_bandwidth_downgrade_time = AAZStrType(
                serialized_name="nextAllowedProvisionedBandwidthDowngradeTime",
                flags={"read_only": True},
            )
            properties.next_allowed_provisioned_iops_downgrade_time = AAZStrType(
                serialized_name="nextAllowedProvisionedIopsDowngradeTime",
                flags={"read_only": True},
            )
            properties.next_allowed_quota_downgrade_time = AAZStrType(
                serialized_name="nextAllowedQuotaDowngradeTime",
                flags={"read_only": True},
            )
            properties.provisioned_bandwidth_mibps = AAZIntType(
                serialized_name="provisionedBandwidthMibps",
            )
            properties.provisioned_iops = AAZIntType(
                serialized_name="provisionedIops",
            )
            properties.remaining_retention_days = AAZIntType(
                serialized_name="remainingRetentionDays",
                flags={"read_only": True},
            )
            properties.root_squash = AAZStrType(
                serialized_name="rootSquash",
            )
            properties.share_quota = AAZIntType(
                serialized_name="shareQuota",
            )
            properties.share_usage_bytes = AAZIntType(
                serialized_name="shareUsageBytes",
                flags={"read_only": True},
            )
            properties.signed_identifiers = AAZListType(
                serialized_name="signedIdentifiers",
            )
            properties.snapshot_time = AAZStrType(
                serialized_name="snapshotTime",
                flags={"read_only": True},
            )
            properties.version = AAZStrType(
                flags={"read_only": True},
            )

            file_share_paid_bursting = cls._schema_on_200.properties.file_share_paid_bursting
            file_share_paid_bursting.paid_bursting_enabled = AAZBoolType(
                serialized_name="paidBurstingEnabled",
            )
            file_share_paid_bursting.paid_bursting_max_bandwidth_mibps = AAZIntType(
                serialized_name="paidBurstingMaxBandwidthMibps",
            )
            file_share_paid_bursting.paid_bursting_max_iops = AAZIntType(
                serialized_name="paidBurstingMaxIops",
            )

            metadata = cls._schema_on_200.properties.metadata
            metadata.Element = AAZStrType()

            signed_identifiers = cls._schema_on_200.properties.signed_identifiers
            signed_identifiers.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.signed_identifiers.Element
            _element.access_policy = AAZObjectType(
                serialized_name="accessPolicy",
            )
            _element.id = AAZStrType()

            access_policy = cls._schema_on_200.properties.signed_identifiers.Element.access_policy
            access_policy.expiry_time = AAZStrType(
                serialized_name="expiryTime",
            )
            access_policy.permission = AAZStrType()
            access_policy.start_time = AAZStrType(
                serialized_name="startTime",
            )

            return cls._schema_on_200


class _ShowHelper:
    """Helper class for Show"""


__all__ = ["Show"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=too-many-locals, too-many-statements too-many-return-statements simplifiable-if-expression

from azure.cli.core.aaz._base import has_value
from azure.mgmt.cdn.models import (RequestMethodOperator, RuleIsCompressionEnabled)
from azure.mgmt.core.tools import is_valid_resource_id


def create_condition(match_variable,
                     operator, match_values=None, selector=None, negate_condition=None, transforms=None):
    condition = None
    if match_variable == 'RemoteAddress':
        condition = {
            "remote_address": {
                "parameters": {
                    "type_name": "DeliveryRuleRemoteAddressConditionParameters",
                    "match_values": match_values,
                    "negate_condition": negate_condition,
                    "operator": operator,
                    "transforms": transforms
                }
            }
        }
    if match_variable == 'RequestMethod':
        condition = {
            "request_method": {
                "parameters": {
                    "type_name": "DeliveryRuleRequestMethodConditionParameters",
                    "match_values": match_values,
                    "negate_condition": negate_condition,
                    "operator": operator if has_value(operator) else RequestMethodOperator.EQUAL,
                }
            }
        }
    if match_variable == 'QueryString':
        condition = {
            "query_string": {
                "parameters": {
                    "type_name": "DeliveryRuleQueryStringConditionParameters",
                    "match_values": match_values,
                    "negate_condition": negate_condition,
                    "operator": operator,
                    "transforms": transforms
                }
            }
        }
    if match_variable == 'PostArgs':
        condition = {
            "post_args": {
                "parameters": {
                    "type_name": "DeliveryRulePostArgsConditionParameters",
                    "match_values": match_values,
                    "negate_condition": negate_condition,
                    "operator": operator,
                    "transforms": transforms,
                    "selector": selector
                }
            }
        }
    if match_variable == 'RequestHeader':
        condition = {
            "request_header": {
                "parameters": {
                    "type_name": "DeliveryRuleRequestHeaderConditionParameters",
                    "match_values": match_values,
                    "negate_condition": negate_condition,
                    "operator": operator,
                    "transforms": transforms,
                    "selector": selector
                }
            }
        }
    if match_variable == 'RequestUri':
        condition = {
            "request_uri": {
                "parameters": {
                    "type_name": "DeliveryRuleRequestUriConditionParameters",
                    "match_values": match_values,
                    "negate_condition": negate_condition,
                    "operator": operator,
                    "transforms": transforms
                }
            }
        }
    if match_variable == 'RequestBody':
        condition = {
            "request_body": {
                "parameters": {
                    "type_name": "DeliveryRuleRequestBodyConditionParameters",
                    "match_values": match_values,
                    "negate_condition": negate_condition,
                    "operator": operator,
                    "transforms": transforms
                }
            }
        }
    if match_variable == 'RequestScheme':
        condition = {
            "request_scheme": {
                "parameters": {
                    "type_name": "DeliveryRuleRequestSchemeConditionParameters",
                    "match_values": match_values,
                    "negate_condition": negate_condition,
                    "operator": operator if has_value(operator) else RequestMethodOperator.EQUAL
                }
            }
        }
    if match_variable == 'UrlPath':
        condition = {
            "url_path": {
                "parameters": {
                    "type_name": "DeliveryRuleUrlPathMatchConditionParameters",
                    "match_values": match_values,
                    "negate_condition": negate_condition,
                    "operator": operator,
                    "transforms": transforms
                }
            }
        }
    if match_variable == 'UrlFileExtension':
        condition = {
            "url_file_extension": {
                "parameters": {
                    "type_name": "DeliveryRuleUrlFileExtensionMatchConditionParameters",
                    "match_values": match_values,
                    "negate_condition": negate_condition,
                    "operator": operator,
                    "transforms": transforms
                }
            }
        }
    if match_variable == 'UrlFileName':
        condition = {
            "url_file_name": {
                "parameters": {
                    "type_name": "DeliveryRuleUrlFilenameConditionParameters",
                    "match_values": match_values,
                    "negate_condition": negate_condition,
                    "operator": operator,
                    "transforms": transforms
                }
            }
        }
    if match_variable == 'HttpVersion':
        condition = {
            "http_version": {
                "parameters": {
                    "type_name": "DeliveryRuleHttpVersionConditionParameters",
                    "match_values": match_values,
                    "negate_condition": negate_condition,
                    "operator": operator,
                    "transforms": transforms
                }
            }
        }
    if match_variable == 'IsDevice':
        condition = {
            "is_device": {
                "parameters": {
                    "type_name": "DeliveryRuleIsDeviceConditionParameters",
                    "match_values": match_values,
                    "negate_condition": negate_condition,
                    "operator": operator,
                    "transforms": transforms
                }
            }
        }
    if match_variable == 'Cookies':
        condition = {
            "cookies": {
                "parameters": {
                    "type_name": "DeliveryRuleCookiesConditionParameters",
                    "match_values": match_values,
                    "negate_condition": negate_condition,
                    "operator": operator,
                    "transforms": transforms,
                    "selector": selector
                }
            }
        }
    if match_variable == 'SocketAddr':
        condition = {
            "socket_addr": {
                "parameters": {
                    "type_name": "DeliveryRuleSocketAddrConditionParameters",
                    "match_values": match_values,
                    "negate_condition": negate_condition,
                    "operator": operator,
                    "transforms": transforms
                }
            }
        }
    if match_variable == 'ClientPort':
        condition = {
            "client_port": {
                "parameters": {
                    "type_name": "DeliveryRuleClientPortConditionParameters",
                    "match_values": match_values,
                    "negate_condition": negate_condition,
                    "operator": operator,
                    "transforms": transforms
                }
            }
        }
    if match_variable == 'ServerPort':
        condition = {
            "server_port": {
                "parameters": {
                    "type_name": "DeliveryRuleServerPortConditionParameters",
                    "match_values": match_values,
                    "negate_condition": negate_condition,
                    "operator": operator,
                    "transforms": transforms
                }
            }
        }
    if match_variable == 'HostName':
        condition = {
            "host_name": {
                "parameters": {
                    "type_name": "DeliveryRuleHostNameConditionParameters",
                    "match_values": match_values,
                    "negate_condition": negate_condition,
                    "operator": operator,
                    "transforms": transforms
                }
            }
        }
    if match_variable == 'SslProtocol':
        condition = {
            "ssl_protocol": {
                "parameters": {
                    "type_name": "DeliveryRuleSslProtocolConditionParameters",
                    "match_values": match_values,
                    "negate_condition": negate_condition,
                    "operator": operator,
                    "transforms": transforms,
                }
            }
        }
    return condition


def create_action(action_name, cache_behavior=None, cache_duration=None, header_action=None,
                  header_name=None, header_value=None, query_string_behavior=None, query_parameters=None,
                  redirect_type=None, redirect_protocol=None, custom_hostname=None, custom_path=None,
                  custom_querystring=None, custom_fragment=None, source_pattern=None, destination=None,
                  preserve_unmatched_path=None, sub_id=None, resource_group=None, profile_name=None,
                  endpoint_name=None, origin_group=None, query_string_caching_behavior=None,
                  enable_compression=None, enable_caching=None, forwarding_protocol=None):
    action = None
    if action_name == "CacheExpiration":
        action = {
            "cache_expiration": {
                "parameters": {
                    "type_name": "DeliveryRuleCacheExpirationActionParameters",
                    "cache_behavior": cache_behavior,
                    "cache_duration": cache_duration,
                    "cache_type": "All"
                }
            }
        }
        return action
    if action_name in ('RequestHeader', 'ModifyRequestHeader'):
        action = {
            "modify_request_header": {
                "parameters": {
                    "type_name": "DeliveryRuleHeaderActionParameters",
                    "header_action": header_action,
                    "header_name": header_name,
                    "value": header_value
                }
            }
        }
        return action
    if action_name in ('ResponseHeader', 'ModifyResponseHeader'):
        action = {
            "modify_response_header": {
                "parameters": {
                    "type_name": "DeliveryRuleHeaderActionParameters",
                    "header_action": header_action,
                    "header_name": header_name,
                    "value": header_value
                }
            }
        }
        return action
    if action_name == "CacheKeyQueryString":
        action = {
            "cache_key_query_string": {
                "parameters": {
                    "type_name": "DeliveryRuleCacheKeyQueryStringBehaviorActionParameters",
                    "query_string_behavior": query_string_behavior,
                    "query_parameters": query_parameters
                }
            }
        }
        return action
    if action_name == "UrlRedirect":
        action = {
            "url_redirect": {
                "parameters": {
                    "type_name": "DeliveryRuleUrlRedirectActionParameters",
                    "custom_fragment": custom_fragment,
                    "custom_hostname": custom_hostname,
                    "custom_path": custom_path,
                    "custom_querystring": custom_querystring,
                    "destination_protocol": redirect_protocol,
                    "redirect_type": redirect_type
                }
            }
        }
        return action
    if action_name == "UrlRewrite":
        action = {
            "url_rewrite": {
                "parameters": {
                    "type_name": "DeliveryRuleUrlRewriteActionParameters",
                    "destination": destination,
                    "preserve_unmatched_path": preserve_unmatched_path,
                    "source_pattern": source_pattern
                }
            }
        }
        return action
    if action_name == "OriginGroupOverride":
        formatetd_origin_group = None
        if has_value(origin_group):
            try:
                formatetd_origin_group = origin_group.to_serialized_data()
            except AttributeError:
                formatetd_origin_group = origin_group
        else:
            formatetd_origin_group = None

        if not is_valid_resource_id(formatetd_origin_group):
            # Ideally we should use resource_id but Auzre FrontDoor portal extension has some case-sensitive issues
            # that prevent it from displaying correctly in portal.
            formatetd_origin_group = f'/subscriptions/{sub_id}/resourcegroups/{resource_group}' \
                f'/providers/Microsoft.Cdn/profiles/{profile_name}/endpoints/{endpoint_name}' \
                f'/origingroups/{formatetd_origin_group}'
        action = {
            "origin_group_override": {
                "parameters": {
                    "type_name": "DeliveryRuleOriginGroupOverrideActionParameters",
                    "origin_group": {
                        "id": formatetd_origin_group
                    }
                }
            }
        }
        return action
    if action_name == "RouteConfigurationOverride":
        origin_group_override = None
        formatetd_origin_group = None
        if has_value(origin_group):
            try:
                formatetd_origin_group = origin_group.to_serialized_data()
            except AttributeError:
                formatetd_origin_group = origin_group
        else:
            formatetd_origin_group = None
        if formatetd_origin_group is not None:
            if is_valid_resource_id(formatetd_origin_group):
                origin_group_override = {
                    "origin_group": {
                        "id": formatetd_origin_group
                    },
                    "forwarding_protocol": forwarding_protocol
                }
            else:
                origin_group_refernce = f'/subscriptions/{sub_id}/resourcegroups/' \
                    f'{resource_group}/providers/Microsoft.Cdn/profiles/{profile_name}/' \
                    f'origingroups/{formatetd_origin_group}'
                origin_group_override = {
                    "origin_group": {
                        "id": origin_group_refernce
                    },
                    "forwarding_protocol": forwarding_protocol
                }
        action = {
            "route_configuration_override": {
                "parameters": {
                    "type_name": "DeliveryRuleRouteConfigurationOverrideActionParameters",
                    "origin_group_override": origin_group_override,
                    "cache_configuration": {
                        "query_string_caching_behavior": query_string_caching_behavior,
                        "query_parameters": query_parameters,
                        "cache_behavior": cache_behavior,
                        "cache_duration": cache_duration,
                        "is_compression_enabled": RuleIsCompressionEnabled.ENABLED.value if enable_compression
                        else RuleIsCompressionEnabled.DISABLED.value
                    } if enable_caching else None
                },
            }
        }
        return action


def create_actions_from_existing(existing_actions):
    parsed_actions = []
    for action in existing_actions:
        name = action['name']
        para = action['parameters']
        if name == 'CacheExpiration':
            parsed_actions.append(create_action(name, cache_behavior=para['cacheBehavior']
                                                if 'cacheBehavior' in para else None,
                                                cache_duration=para['cacheDuration']
                                                if 'cacheDuration' in para else None))
        if name == 'ModifyRequestHeader':
            parsed_actions.append(create_action(name, header_action=para['headerAction']
                                                if 'headerAction' in para else None,
                                                header_name=para['headerName']
                                                if 'headerName' in para else None,
                                                header_value=para['value']
                                                if 'value' in para else None))
        if name == 'ModifyResponseHeader':
            parsed_actions.append(create_action(name, header_action=para['headerAction']
                                                if 'headerAction' in para else None,
                                                header_name=para['headerName']
                                                if 'headerName' in para else None,
                                                header_value=para['value']
                                                if 'value' in para else None))
        if name == 'CacheKeyQueryString':
            parsed_actions.append(create_action(name, query_string_behavior=para['queryStringBehavior']
                                                if 'queryStringBehavior' in para else None,
                                                query_parameters=para['queryParameters']
                                                if 'queryParameters' in para else None))
        if name == 'UrlRedirect':
            parsed_actions.append(create_action(name, custom_fragment=para['customFragment']
                                                if 'customFragment' in para else None,
                                                custom_hostname=para['customHostname']
                                                if 'customHostname' in para else None,
                                                custom_path=para['customPath']
                                                if 'customPath' in para else None,
                                                custom_querystring=para['customQueryString']
                                                if 'customQueryString' in para else None,
                                                redirect_protocol=para['destinationProtocol']
                                                if 'destinationProtocol' in para else None,
                                                redirect_type=para['redirectType']
                                                if 'redirectType' in para else None))
        if name == 'UrlRewrite':
            parsed_actions.append(create_action(name, destination=para['destination']
                                                if 'destination' in para else None,
                                                preserve_unmatched_path=para['preserveUnmatchedPath']
                                                if 'preserveUnmatchedPath' in para else None,
                                                source_pattern=para['sourcePattern']
                                                if 'sourcePattern' in para else None))
        if name == 'OriginGroupOverride':
            parsed_actions.append(create_action(name, origin_group=para['originGroup']['id']
                                                if 'originGroup' in para else None))
        if name == 'RouteConfigurationOverride':
            enable_caching = False
            if 'cacheConfiguration' in para:
                enable_caching = True
            parsed_actions.append(create_action(name,
                                                origin_group=para['originGroupOverride']['originGroup']['id']
                                                if 'originGroupOverride' in para and
                                                'originGroup' in para['originGroupOverride']['originGroup']
                                                else None,
                                                forwarding_protocol=para['originGroupOverride']['forwardingProtocol']
                                                if 'originGroupOverride' in para and
                                                'forwardingProtocol' in para['originGroupOverride']
                                                else None,
                                                query_string_caching_behavior=para
                                                ['cacheConfiguration']['queryStringCachingBehavior']
                                                if 'cacheConfiguration' in para and
                                                'queryStringCachingBehavior' in para['cacheConfiguration']
                                                else None,
                                                query_parameters=para['cacheConfiguration']['queryParameters']
                                                if 'cacheConfiguration' in para and
                                                'queryParameters' in para['cacheConfiguration']
                                                else None,
                                                cache_behavior=para['cacheConfiguration']['cacheBehavior']
                                                if 'cacheConfiguration' in para and
                                                'cacheBehavior' in para['cacheConfiguration']
                                                else None,
                                                cache_duration=para['cacheConfiguration']['cacheDuration']
                                                if 'cacheConfiguration' in para and
                                                'cacheDuration' in para['cacheConfiguration']
                                                else None,
                                                enable_compression=True
                                                if 'cacheConfiguration' in para and
                                                'isCompressionEnabled' in para['cacheConfiguration'] and
                                                para['cacheConfiguration']['isCompressionEnabled'] ==
                                                RuleIsCompressionEnabled.ENABLED.value
                                                else False,
                                                enable_caching=enable_caching))
    if len(parsed_actions) == 0:
        return []
    return parsed_actions


def create_conditions_from_existing(existing_conditions):
    parsed_conditions = []
    for condition in existing_conditions:
        parsed_conditions.append(create_condition(
            condition['name'], match_values=condition['parameters']['matchValues']
            if 'matchValues' in condition['parameters'] else None,
            negate_condition=condition['parameters']['negateCondition']
            if 'negateCondition' in condition['parameters'] else None,
            operator=condition['parameters']['operator']
            if 'operator' in condition['parameters'] else None,
            transforms=condition['parameters']['transforms']
            if 'transforms' in condition['parameters'] else None,
            selector=condition['parameters']['selector']
            if 'selector' in condition['parameters'] else None))
    if len(parsed_conditions) == 0:
        return []
    return parsed_conditions


def create_delivery_policy_from_existing(existing_policy):
    parsed_policy = {
        'description': 'default_policy',
        'rules': []
    }

    if 'rules' in existing_policy:
        for rule in existing_policy['rules']:
            parsed_rule = {
                'name': rule['name'] if 'name' in rule else None,
                'order': rule['order'],
                'conditions': create_conditions_from_existing(rule['conditions']),
                'actions': create_actions_from_existing(rule['actions'])
            }
            parsed_policy['rules'].append(parsed_rule)

    return parsed_policy



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


class ListByType(AAZCommand):
    """Get the record sets of a specified type in a DNS zone.
    """

    _aaz_info = {
        "version": "2023-07-01-preview",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.network/dnszones/{}/{}", "2023-07-01-preview"],
        ]
    }

    AZ_SUPPORT_PAGINATION = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_paging(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.record_type = AAZStrArg(
            options=["--record-type"],
            help="The type of record sets to enumerate.",
            required=True,
            enum={"A": "A", "AAAA": "AAAA", "CAA": "CAA", "CNAME": "CNAME", "DS": "DS", "MX": "MX", "NAPTR": "NAPTR", "NS": "NS", "PTR": "PTR", "SOA": "SOA", "SRV": "SRV", "TLSA": "TLSA", "TXT": "TXT"},
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.zone_name = AAZStrArg(
            options=["-z", "--zone-name"],
            help="The name of the DNS zone (without a terminating dot).",
            required=True,
        )
        _args_schema.recordsetnamesuffix = AAZStrArg(
            options=["--recordsetnamesuffix"],
            help="The suffix label of the record set name that has to be used to filter the record set enumerations. If this parameter is specified, Enumeration will return only records that end with `.<recordSetNameSuffix>`",
        )
        _args_schema.top = AAZIntArg(
            options=["--top"],
            help="The maximum number of record sets to return. If not specified, returns up to 100 record sets.",
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.RecordSetsListByType(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance.value, client_flatten=True)
        next_link = self.deserialize_output(self.ctx.vars.instance.next_link)
        return result, next_link

    class RecordSetsListByType(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/dnsZones/{zoneName}/{recordType}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "recordType", self.ctx.args.record_type,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
                **self.serialize_url_param(
                    "zoneName", self.ctx.args.zone_name,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "$recordsetnamesuffix", self.ctx.args.recordsetnamesuffix,
                ),
                **self.serialize_query_param(
                    "$top", self.ctx.args.top,
                ),
                **self.serialize_query_param(
                    "api-version", "2023-07-01-preview",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.next_link = AAZStrType(
                serialized_name="nextLink",
                flags={"read_only": True},
            )
            _schema_on_200.value = AAZListType()

            value = cls._schema_on_200.value
            value.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element
            _element.etag = AAZStrType()
            _element.id = AAZStrType(
                flags={"read_only": True},
            )
            _element.name = AAZStrType(
                flags={"read_only": True},
            )
            _element.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _element.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.value.Element.properties
            properties.aaaa_records = AAZListType(
                serialized_name="AAAARecords",
            )
            properties.a_records = AAZListType(
                serialized_name="ARecords",
            )
            properties.cname_record = AAZObjectType(
                serialized_name="CNAMERecord",
            )
            properties.ds_records = AAZListType(
                serialized_name="DSRecords",
            )
            properties.mx_records = AAZListType(
                serialized_name="MXRecords",
            )
            properties.naptr_records = AAZListType(
                serialized_name="NAPTRRecords",
            )
            properties.ns_records = AAZListType(
                serialized_name="NSRecords",
            )
            properties.ptr_records = AAZListType(
                serialized_name="PTRRecords",
            )
            properties.soa_record = AAZObjectType(
                serialized_name="SOARecord",
            )
            properties.srv_records = AAZListType(
                serialized_name="SRVRecords",
            )
            properties.tlsa_records = AAZListType(
                serialized_name="TLSARecords",
            )
            properties.ttl = AAZIntType(
                serialized_name="TTL",
            )
            properties.txt_records = AAZListType(
                serialized_name="TXTRecords",
            )
            properties.caa_records = AAZListType(
                serialized_name="caaRecords",
            )
            properties.fqdn = AAZStrType(
                flags={"read_only": True},
            )
            properties.metadata = AAZDictType()
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.target_resource = AAZObjectType(
                serialized_name="targetResource",
            )
            _ListByTypeHelper._build_schema_sub_resource_read(properties.target_resource)
            properties.traffic_management_profile = AAZObjectType(
                serialized_name="trafficManagementProfile",
            )
            _ListByTypeHelper._build_schema_sub_resource_read(properties.traffic_management_profile)

            aaaa_records = cls._schema_on_200.value.Element.properties.aaaa_records
            aaaa_records.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.aaaa_records.Element
            _element.ipv6_address = AAZStrType(
                serialized_name="ipv6Address",
            )

            a_records = cls._schema_on_200.value.Element.properties.a_records
            a_records.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.a_records.Element
            _element.ipv4_address = AAZStrType(
                serialized_name="ipv4Address",
            )

            cname_record = cls._schema_on_200.value.Element.properties.cname_record
            cname_record.cname = AAZStrType()

            ds_records = cls._schema_on_200.value.Element.properties.ds_records
            ds_records.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.ds_records.Element
            _element.algorithm = AAZIntType()
            _element.digest = AAZObjectType()
            _element.key_tag = AAZIntType(
                serialized_name="keyTag",
            )

            digest = cls._schema_on_200.value.Element.properties.ds_records.Element.digest
            digest.algorithm_type = AAZIntType(
                serialized_name="algorithmType",
            )
            digest.value = AAZStrType()

            mx_records = cls._schema_on_200.value.Element.properties.mx_records
            mx_records.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.mx_records.Element
            _element.exchange = AAZStrType()
            _element.preference = AAZIntType()

            naptr_records = cls._schema_on_200.value.Element.properties.naptr_records
            naptr_records.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.naptr_records.Element
            _element.flags = AAZStrType()
            _element.order = AAZIntType()
            _element.preference = AAZIntType()
            _element.regexp = AAZStrType()
            _element.replacement = AAZStrType()
            _element.services = AAZStrType()

            ns_records = cls._schema_on_200.value.Element.properties.ns_records
            ns_records.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.ns_records.Element
            _element.nsdname = AAZStrType()

            ptr_records = cls._schema_on_200.value.Element.properties.ptr_records
            ptr_records.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.ptr_records.Element
            _element.ptrdname = AAZStrType()

            soa_record = cls._schema_on_200.value.Element.properties.soa_record
            soa_record.email = AAZStrType()
            soa_record.expire_time = AAZIntType(
                serialized_name="expireTime",
            )
            soa_record.host = AAZStrType()
            soa_record.minimum_ttl = AAZIntType(
                serialized_name="minimumTTL",
            )
            soa_record.refresh_time = AAZIntType(
                serialized_name="refreshTime",
            )
            soa_record.retry_time = AAZIntType(
                serialized_name="retryTime",
            )
            soa_record.serial_number = AAZIntType(
                serialized_name="serialNumber",
            )

            srv_records = cls._schema_on_200.value.Element.properties.srv_records
            srv_records.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.srv_records.Element
            _element.port = AAZIntType()
            _element.priority = AAZIntType()
            _element.target = AAZStrType()
            _element.weight = AAZIntType()

            tlsa_records = cls._schema_on_200.value.Element.properties.tlsa_records
            tlsa_records.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.tlsa_records.Element
            _element.cert_association_data = AAZStrType(
                serialized_name="certAssociationData",
            )
            _element.matching_type = AAZIntType(
                serialized_name="matchingType",
            )
            _element.selector = AAZIntType()
            _element.usage = AAZIntType()

            txt_records = cls._schema_on_200.value.Element.properties.txt_records
            txt_records.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.txt_records.Element
            _element.value = AAZListType()

            value = cls._schema_on_200.value.Element.properties.txt_records.Element.value
            value.Element = AAZStrType()

            caa_records = cls._schema_on_200.value.Element.properties.caa_records
            caa_records.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.caa_records.Element
            _element.flags = AAZIntType()
            _element.tag = AAZStrType()
            _element.value = AAZStrType()

            metadata = cls._schema_on_200.value.Element.properties.metadata
            metadata.Element = AAZStrType()

            return cls._schema_on_200


class _ListByTypeHelper:
    """Helper class for ListByType"""

    _schema_sub_resource_read = None

    @classmethod
    def _build_schema_sub_resource_read(cls, _schema):
        if cls._schema_sub_resource_read is not None:
            _schema.id = cls._schema_sub_resource_read.id
            return

        cls._schema_sub_resource_read = _schema_sub_resource_read = AAZObjectType()

        sub_resource_read = _schema_sub_resource_read
        sub_resource_read.id = AAZStrType()

        _schema.id = cls._schema_sub_resource_read.id


__all__ = ["ListByType"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import tarfile
import os
import re
import codecs
import requests
from knack.log import get_logger
from knack.util import CLIError
from azure.core.exceptions import HttpResponseError
from azure.cli.core.profiles import ResourceType, get_sdk
from ._constants import TASK_VALID_VSTS_URLS

logger = get_logger(__name__)


def upload_source_code(cmd, client,
                       registry_name,
                       resource_group_name,
                       source_location,
                       tar_file_path,
                       docker_file_path,
                       docker_file_in_tar):
    _pack_source_code(source_location,
                      tar_file_path,
                      docker_file_path,
                      docker_file_in_tar)

    size = os.path.getsize(tar_file_path)
    unit = 'GiB'
    for S in ['Bytes', 'KiB', 'MiB', 'GiB']:
        if size < 1024:
            unit = S
            break
        size = size / 1024.0

    logger.warning("Uploading archived source code from '%s'...", tar_file_path)
    upload_url = None
    relative_path = None
    try:
        source_upload_location = client.get_build_source_upload_url(
            resource_group_name, registry_name)
        upload_url = source_upload_location.upload_url
        relative_path = source_upload_location.relative_path
    except (AttributeError, HttpResponseError) as e:
        raise CLIError("Failed to get a SAS URL to upload context. Error: {}".format(e.message))

    if not upload_url:
        raise CLIError("Failed to get a SAS URL to upload context.")

    BlobClient = get_sdk(cmd.cli_ctx, ResourceType.DATA_STORAGE_BLOB, '_blob_client#BlobClient')
    BlobClient = BlobClient.from_blob_url(upload_url, connection_timeout=300)
    with open(tar_file_path, "rb") as data:
        BlobClient.upload_blob(data=data, blob_type="BlockBlob", overwrite=True)
    logger.warning("Sending context ({0:.3f} {1}) to registry: {2}...".format(
        size, unit, registry_name))
    return relative_path


def _pack_source_code(source_location, tar_file_path, docker_file_path, docker_file_in_tar):
    logger.warning("Packing source code into tar to upload...")

    original_docker_file_name = os.path.basename(docker_file_path.replace("\\", os.sep))
    ignore_list, ignore_list_size = _load_dockerignore_file(source_location, original_docker_file_name)
    common_vcs_ignore_list = {'.git', '.gitignore', '.bzr', 'bzrignore', '.hg', '.hgignore', '.svn'}

    def _ignore_check(tarinfo, parent_ignored, parent_matching_rule_index):
        # ignore common vcs dir or file
        if tarinfo.name in common_vcs_ignore_list:
            logger.warning("Excluding '%s' based on default ignore rules", tarinfo.name)
            return True, parent_matching_rule_index

        if ignore_list is None:
            # if .dockerignore doesn't exists, inherit from parent
            # eg, it will ignore the files under .git folder.
            return parent_ignored, parent_matching_rule_index

        for index, item in enumerate(ignore_list):
            # stop checking the remaining rules whose priorities are lower than the parent matching rule
            # at this point, current item should just inherit from parent
            if index >= parent_matching_rule_index:
                break
            if re.match(item.pattern, tarinfo.name):
                logger.debug(".dockerignore: rule '%s' matches '%s'.",
                             item.rule, tarinfo.name)
                return item.ignore, index

        logger.debug(".dockerignore: no rule for '%s'. parent ignore '%s'",
                     tarinfo.name, parent_ignored)
        # inherit from parent
        return parent_ignored, parent_matching_rule_index

    with tarfile.open(tar_file_path, "w:gz") as tar:
        # need to set arcname to empty string as the archive root path
        _archive_file_recursively(tar,
                                  source_location,
                                  arcname="",
                                  parent_ignored=False,
                                  parent_matching_rule_index=ignore_list_size,
                                  ignore_check=_ignore_check)

        # Add the Dockerfile if it's specified.
        # In the case of run, there will be no Dockerfile.
        if docker_file_path:
            docker_file_tarinfo = tar.gettarinfo(
                docker_file_path, docker_file_in_tar)
            with open(docker_file_path, "rb") as f:
                tar.addfile(docker_file_tarinfo, f)


class IgnoreRule:  # pylint: disable=too-few-public-methods
    def __init__(self, rule):

        self.rule = rule
        self.ignore = True
        # ! makes exceptions to exclusions
        if rule.startswith('!'):
            self.ignore = False
            rule = rule[1:]  # remove !
            # load path without leading slash in linux and windows
            # environments (interferes with dockerignore file)
            if rule.startswith('/'):
                rule = rule[1:]  # remove beginning '/'
            # remove trailing slash if included (in parity with docker)
            while rule[-1] == "/":
                rule = rule[:-1]

        self.pattern = "^"
        tokens = rule.split('/')
        token_length = len(tokens)
        for index, token in enumerate(tokens, 1):
            # ** matches any number of directories
            if token == "**":
                self.pattern += ".*"  # treat **/ as **
            else:
                # * matches any sequence of non-seperator characters
                # ? matches any single non-seperator character
                # . matches dot character
                self.pattern += token.replace(
                    "*", "[^/]*").replace("?", "[^/]").replace(".", "\\.")
            if index < token_length:
                self.pattern += "/"  # add back / if it's not the last
        self.pattern += "$"


def _load_dockerignore_file(source_location, original_docker_file_name):
    # reference: https://docs.docker.com/engine/reference/builder/#dockerignore-file
    docker_ignore_file = os.path.join(source_location, ".dockerignore")
    docker_ignore_file_override = None
    if original_docker_file_name != "Dockerfile":
        docker_ignore_file_override = os.path.join(
            source_location, "{}.dockerignore".format(original_docker_file_name))
        if os.path.exists(docker_ignore_file_override):
            logger.warning("Overriding .dockerignore with %s", docker_ignore_file_override)
            docker_ignore_file = docker_ignore_file_override

    if not os.path.exists(docker_ignore_file):
        return None, 0

    encoding = "utf-8"
    header = open(docker_ignore_file, "rb").read(len(codecs.BOM_UTF8))
    if header.startswith(codecs.BOM_UTF8):
        encoding = "utf-8-sig"

    ignore_list = []
    if docker_ignore_file == docker_ignore_file_override:
        ignore_list.append(IgnoreRule(".dockerignore"))

    for line in open(docker_ignore_file, 'r', encoding=encoding).readlines():
        rule = line.rstrip()

        # skip empty line and comment
        if not rule or rule.startswith('#'):
            continue

        # the ignore rule at the end has higher priority
        ignore_list = [IgnoreRule(rule)] + ignore_list

    return ignore_list, len(ignore_list)


def _archive_file_recursively(tar, name, arcname, parent_ignored, parent_matching_rule_index, ignore_check):
    # create a TarInfo object from the file
    tarinfo = tar.gettarinfo(name, arcname)

    if tarinfo is None:
        raise CLIError("tarfile: unsupported type {}".format(name))

    # check if the file/dir is ignored
    ignored, matching_rule_index = ignore_check(
        tarinfo, parent_ignored, parent_matching_rule_index)

    if not ignored:
        # append the tar header and data to the archive
        if tarinfo.isreg():
            with open(name, "rb") as f:
                tar.addfile(tarinfo, f)
        else:
            tar.addfile(tarinfo)

    # even the dir is ignored, its child items can still be included, so continue to scan
    if tarinfo.isdir():
        for f in os.listdir(name):
            _archive_file_recursively(tar, os.path.join(name, f), os.path.join(arcname, f),
                                      parent_ignored=ignored, parent_matching_rule_index=matching_rule_index,
                                      ignore_check=ignore_check)


def check_remote_source_code(source_location):
    lower_source_location = source_location.lower()

    # git
    if lower_source_location.startswith("git@") or lower_source_location.startswith("git://"):
        return source_location

    # http
    if lower_source_location.startswith("https://") or lower_source_location.startswith("http://") \
       or lower_source_location.startswith("github.com/"):
        isVSTS = any(url in lower_source_location for url in TASK_VALID_VSTS_URLS)
        if isVSTS or re.search(r"\.git(?:#.+)?$", lower_source_location):
            # git url must contain ".git" or be from VSTS/Azure DevOps.
            # This is because Azure DevOps doesn't follow the standard git server convention of putting
            # .git at the end of their URLs, so we have to special case them.
            return source_location
        if not lower_source_location.startswith("github.com/"):
            # Others are tarball
            if requests.head(source_location).status_code < 400:
                return source_location
            raise CLIError("'{}' doesn't exist.".format(source_location))

    # oci
    if lower_source_location.startswith("oci://"):
        return source_location
    raise CLIError("'{}' doesn't exist.".format(source_location))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "monitor autoscale update",
)
class Update(AAZCommand):
    """Update an autoscale setting.

    For more information on autoscaling, visit: https://learn.microsoft.com/azure/monitoring-and-diagnostics/monitoring-understanding-autoscale-settings.

    :example: Update autoscale settings to use a fixed 3 instances by default.
        az monitor autoscale update -g myrg -n autoscale-name --count 3

    :example: Update autoscale settings to remove an email notification.
        az monitor autoscale update -g myrg -n autoscale-name --remove-action email bob@contoso.com

    :example: Update autoscale settings.
        az monitor autoscale update --count 3 --email-administrator true --enabled true --max-count 5 --min-count 2 --name MyAutoscaleSettings --resource-group MyResourceGroup --tags key[=value]
    """

    _aaz_info = {
        "version": "2022-10-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.insights/autoscalesettings/{}", "2022-10-01"],
        ]
    }

    AZ_SUPPORT_GENERIC_UPDATE = True

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.autoscale_name = AAZStrArg(
            options=["-n", "--name", "--autoscale-name"],
            help="The autoscale setting name.",
            required=True,
            id_part="name",
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.enabled = AAZBoolArg(
            options=["--enabled"],
            help="the enabled flag. Specifies whether automatic scaling is enabled for the resource. The default value is 'false'.",
            nullable=True,
        )
        _args_schema.tags = AAZDictArg(
            options=["--tags"],
            help="Gets or sets a list of key value pairs that describe the resource. These tags can be used in viewing and grouping this resource (across resource groups). A maximum of 15 tags can be provided for a resource. Each tag must have a key no greater in length than 128 characters and a value no greater in length than 256 characters.",
            nullable=True,
        )

        tags = cls._args_schema.tags
        tags.Element = AAZStrArg(
            nullable=True,
        )

        # define Arg Group "Predictive Policy"

        _args_schema = cls._args_schema
        _args_schema.scale_look_ahead_time = AAZDurationArg(
            options=["--scale-look-ahead-time"],
            arg_group="Predictive Policy",
            help="the amount of time to specify by which instances are launched in advance. It must be between 1 minute and 60 minutes in ISO 8601 format (for example, 100 days would be P100D).",
            nullable=True,
        )
        _args_schema.scale_mode = AAZStrArg(
            options=["--scale-mode"],
            arg_group="Predictive Policy",
            help="the predictive autoscale mode",
            enum={"Disabled": "Disabled", "Enabled": "Enabled", "ForecastOnly": "ForecastOnly"},
        )

        # define Arg Group "Properties"

        _args_schema = cls._args_schema
        _args_schema.notifications = AAZListArg(
            options=["--notifications"],
            arg_group="Properties",
            help="the collection of notifications.",
            nullable=True,
        )
        _args_schema.profiles = AAZListArg(
            options=["--profiles"],
            arg_group="Properties",
            help="the collection of automatic scaling profiles that specify different scaling parameters for different time periods. A maximum of 20 profiles can be specified.",
        )
        _args_schema.target_resource_location = AAZStrArg(
            options=["--target-resource-location"],
            arg_group="Properties",
            help="the location of the resource that the autoscale setting should be added to.",
            nullable=True,
        )
        _args_schema.target_resource_uri = AAZStrArg(
            options=["--target-resource-uri"],
            arg_group="Properties",
            help="the resource identifier of the resource that the autoscale setting should be added to.",
            nullable=True,
        )

        notifications = cls._args_schema.notifications
        notifications.Element = AAZObjectArg(
            nullable=True,
        )

        _element = cls._args_schema.notifications.Element
        _element.email = AAZObjectArg(
            options=["email"],
            help="the email notification.",
            nullable=True,
        )
        _element.operation = AAZStrArg(
            options=["operation"],
            help="the operation associated with the notification and its value must be \"scale\"",
            enum={"Scale": "Scale"},
        )
        _element.webhooks = AAZListArg(
            options=["webhooks"],
            help="the collection of webhook notifications.",
            nullable=True,
        )

        email = cls._args_schema.notifications.Element.email
        email.custom_emails = AAZListArg(
            options=["custom-emails"],
            help="the custom e-mails list. This value can be null or empty, in which case this attribute will be ignored.",
            nullable=True,
        )
        email.send_to_subscription_administrator = AAZBoolArg(
            options=["send-to-subscription-administrator"],
            help="a value indicating whether to send email to subscription administrator.",
            nullable=True,
        )
        email.send_to_subscription_co_administrators = AAZBoolArg(
            options=["send-to-subscription-co-administrators"],
            help="a value indicating whether to send email to subscription co-administrators.",
            nullable=True,
        )

        custom_emails = cls._args_schema.notifications.Element.email.custom_emails
        custom_emails.Element = AAZStrArg(
            nullable=True,
        )

        webhooks = cls._args_schema.notifications.Element.webhooks
        webhooks.Element = AAZObjectArg(
            nullable=True,
        )

        _element = cls._args_schema.notifications.Element.webhooks.Element
        _element.properties = AAZDictArg(
            options=["properties"],
            help="a property bag of settings. This value can be empty.",
            nullable=True,
        )
        _element.service_uri = AAZStrArg(
            options=["service-uri"],
            help="the service address to receive the notification.",
            nullable=True,
        )

        properties = cls._args_schema.notifications.Element.webhooks.Element.properties
        properties.Element = AAZStrArg(
            nullable=True,
        )

        profiles = cls._args_schema.profiles
        profiles.Element = AAZObjectArg(
            nullable=True,
        )

        _element = cls._args_schema.profiles.Element
        _element.capacity = AAZObjectArg(
            options=["capacity"],
            help="the number of instances that can be used during this profile.",
        )
        _element.fixed_date = AAZObjectArg(
            options=["fixed-date"],
            help="the specific date-time for the profile. This element is not used if the Recurrence element is used.",
            nullable=True,
        )
        _element.name = AAZStrArg(
            options=["name"],
            help="the name of the profile.",
        )
        _element.recurrence = AAZObjectArg(
            options=["recurrence"],
            help="the repeating times at which this profile begins. This element is not used if the FixedDate element is used.",
            nullable=True,
        )
        _element.rules = AAZListArg(
            options=["rules"],
            help="the collection of rules that provide the triggers and parameters for the scaling action. A maximum of 10 rules can be specified.",
        )

        capacity = cls._args_schema.profiles.Element.capacity
        capacity.default = AAZStrArg(
            options=["default"],
            help="the number of instances that will be set if metrics are not available for evaluation. The default is only used if the current instance count is lower than the default.",
        )
        capacity.maximum = AAZStrArg(
            options=["maximum"],
            help="the maximum number of instances for the resource. The actual maximum number of instances is limited by the cores that are available in the subscription.",
        )
        capacity.minimum = AAZStrArg(
            options=["minimum"],
            help="the minimum number of instances for the resource.",
        )

        fixed_date = cls._args_schema.profiles.Element.fixed_date
        fixed_date.end = AAZDateTimeArg(
            options=["end"],
            help="the end time for the profile in ISO 8601 format.",
        )
        fixed_date.start = AAZDateTimeArg(
            options=["start"],
            help="the start time for the profile in ISO 8601 format.",
        )
        fixed_date.time_zone = AAZStrArg(
            options=["time-zone"],
            help="the timezone of the start and end times for the profile. Some examples of valid time zones are: Dateline Standard Time, UTC-11, Hawaiian Standard Time, Alaskan Standard Time, Pacific Standard Time (Mexico), Pacific Standard Time, US Mountain Standard Time, Mountain Standard Time (Mexico), Mountain Standard Time, Central America Standard Time, Central Standard Time, Central Standard Time (Mexico), Canada Central Standard Time, SA Pacific Standard Time, Eastern Standard Time, US Eastern Standard Time, Venezuela Standard Time, Paraguay Standard Time, Atlantic Standard Time, Central Brazilian Standard Time, SA Western Standard Time, Pacific SA Standard Time, Newfoundland Standard Time, E. South America Standard Time, Argentina Standard Time, SA Eastern Standard Time, Greenland Standard Time, Montevideo Standard Time, Bahia Standard Time, UTC-02, Mid-Atlantic Standard Time, Azores Standard Time, Cape Verde Standard Time, Morocco Standard Time, UTC, GMT Standard Time, Greenwich Standard Time, W. Europe Standard Time, Central Europe Standard Time, Romance Standard Time, Central European Standard Time, W. Central Africa Standard Time, Namibia Standard Time, Jordan Standard Time, GTB Standard Time, Middle East Standard Time, Egypt Standard Time, Syria Standard Time, E. Europe Standard Time, South Africa Standard Time, FLE Standard Time, Turkey Standard Time, Israel Standard Time, Kaliningrad Standard Time, Libya Standard Time, Arabic Standard Time, Arab Standard Time, Belarus Standard Time, Russian Standard Time, E. Africa Standard Time, Iran Standard Time, Arabian Standard Time, Azerbaijan Standard Time, Russia Time Zone 3, Mauritius Standard Time, Georgian Standard Time, Caucasus Standard Time, Afghanistan Standard Time, West Asia Standard Time, Ekaterinburg Standard Time, Pakistan Standard Time, India Standard Time, Sri Lanka Standard Time, Nepal Standard Time, Central Asia Standard Time, Bangladesh Standard Time, N. Central Asia Standard Time, Myanmar Standard Time, SE Asia Standard Time, North Asia Standard Time, China Standard Time, North Asia East Standard Time, Singapore Standard Time, W. Australia Standard Time, Taipei Standard Time, Ulaanbaatar Standard Time, Tokyo Standard Time, Korea Standard Time, Yakutsk Standard Time, Cen. Australia Standard Time, AUS Central Standard Time, E. Australia Standard Time, AUS Eastern Standard Time, West Pacific Standard Time, Tasmania Standard Time, Magadan Standard Time, Vladivostok Standard Time, Russia Time Zone 10, Central Pacific Standard Time, Russia Time Zone 11, New Zealand Standard Time, UTC+12, Fiji Standard Time, Kamchatka Standard Time, Tonga Standard Time, Samoa Standard Time, Line Islands Standard Time",
            nullable=True,
        )

        recurrence = cls._args_schema.profiles.Element.recurrence
        recurrence.frequency = AAZStrArg(
            options=["frequency"],
            help="the recurrence frequency. How often the schedule profile should take effect. This value must be Week, meaning each week will have the same set of profiles. For example, to set a daily schedule, set **schedule** to every day of the week. The frequency property specifies that the schedule is repeated weekly.",
            enum={"Day": "Day", "Hour": "Hour", "Minute": "Minute", "Month": "Month", "None": "None", "Second": "Second", "Week": "Week", "Year": "Year"},
        )
        recurrence.schedule = AAZObjectArg(
            options=["schedule"],
            help="the scheduling constraints for when the profile begins.",
        )

        schedule = cls._args_schema.profiles.Element.recurrence.schedule
        schedule.days = AAZListArg(
            options=["days"],
            help="the collection of days that the profile takes effect on. Possible values are Sunday through Saturday.",
        )
        schedule.hours = AAZListArg(
            options=["hours"],
            help="A collection of hours that the profile takes effect on. Values supported are 0 to 23 on the 24-hour clock (AM/PM times are not supported).",
        )
        schedule.minutes = AAZListArg(
            options=["minutes"],
            help="A collection of minutes at which the profile takes effect at.",
        )
        schedule.time_zone = AAZStrArg(
            options=["time-zone"],
            help="the timezone for the hours of the profile. Some examples of valid time zones are: Dateline Standard Time, UTC-11, Hawaiian Standard Time, Alaskan Standard Time, Pacific Standard Time (Mexico), Pacific Standard Time, US Mountain Standard Time, Mountain Standard Time (Mexico), Mountain Standard Time, Central America Standard Time, Central Standard Time, Central Standard Time (Mexico), Canada Central Standard Time, SA Pacific Standard Time, Eastern Standard Time, US Eastern Standard Time, Venezuela Standard Time, Paraguay Standard Time, Atlantic Standard Time, Central Brazilian Standard Time, SA Western Standard Time, Pacific SA Standard Time, Newfoundland Standard Time, E. South America Standard Time, Argentina Standard Time, SA Eastern Standard Time, Greenland Standard Time, Montevideo Standard Time, Bahia Standard Time, UTC-02, Mid-Atlantic Standard Time, Azores Standard Time, Cape Verde Standard Time, Morocco Standard Time, UTC, GMT Standard Time, Greenwich Standard Time, W. Europe Standard Time, Central Europe Standard Time, Romance Standard Time, Central European Standard Time, W. Central Africa Standard Time, Namibia Standard Time, Jordan Standard Time, GTB Standard Time, Middle East Standard Time, Egypt Standard Time, Syria Standard Time, E. Europe Standard Time, South Africa Standard Time, FLE Standard Time, Turkey Standard Time, Israel Standard Time, Kaliningrad Standard Time, Libya Standard Time, Arabic Standard Time, Arab Standard Time, Belarus Standard Time, Russian Standard Time, E. Africa Standard Time, Iran Standard Time, Arabian Standard Time, Azerbaijan Standard Time, Russia Time Zone 3, Mauritius Standard Time, Georgian Standard Time, Caucasus Standard Time, Afghanistan Standard Time, West Asia Standard Time, Ekaterinburg Standard Time, Pakistan Standard Time, India Standard Time, Sri Lanka Standard Time, Nepal Standard Time, Central Asia Standard Time, Bangladesh Standard Time, N. Central Asia Standard Time, Myanmar Standard Time, SE Asia Standard Time, North Asia Standard Time, China Standard Time, North Asia East Standard Time, Singapore Standard Time, W. Australia Standard Time, Taipei Standard Time, Ulaanbaatar Standard Time, Tokyo Standard Time, Korea Standard Time, Yakutsk Standard Time, Cen. Australia Standard Time, AUS Central Standard Time, E. Australia Standard Time, AUS Eastern Standard Time, West Pacific Standard Time, Tasmania Standard Time, Magadan Standard Time, Vladivostok Standard Time, Russia Time Zone 10, Central Pacific Standard Time, Russia Time Zone 11, New Zealand Standard Time, UTC+12, Fiji Standard Time, Kamchatka Standard Time, Tonga Standard Time, Samoa Standard Time, Line Islands Standard Time",
        )

        days = cls._args_schema.profiles.Element.recurrence.schedule.days
        days.Element = AAZStrArg(
            nullable=True,
        )

        hours = cls._args_schema.profiles.Element.recurrence.schedule.hours
        hours.Element = AAZIntArg(
            nullable=True,
        )

        minutes = cls._args_schema.profiles.Element.recurrence.schedule.minutes
        minutes.Element = AAZIntArg(
            nullable=True,
        )

        rules = cls._args_schema.profiles.Element.rules
        rules.Element = AAZObjectArg(
            nullable=True,
        )

        _element = cls._args_schema.profiles.Element.rules.Element
        _element.metric_trigger = AAZObjectArg(
            options=["metric-trigger"],
            help="the trigger that results in a scaling action.",
        )
        _element.scale_action = AAZObjectArg(
            options=["scale-action"],
            help="the parameters for the scaling action.",
        )

        metric_trigger = cls._args_schema.profiles.Element.rules.Element.metric_trigger
        metric_trigger.dimensions = AAZListArg(
            options=["dimensions"],
            help="List of dimension conditions. For example: [{\"DimensionName\":\"AppName\",\"Operator\":\"Equals\",\"Values\":[\"App1\"]},{\"DimensionName\":\"Deployment\",\"Operator\":\"Equals\",\"Values\":[\"default\"]}].",
            nullable=True,
        )
        metric_trigger.divide_per_instance = AAZBoolArg(
            options=["divide-per-instance"],
            help="a value indicating whether metric should divide per instance.",
            nullable=True,
        )
        metric_trigger.metric_name = AAZStrArg(
            options=["metric-name"],
            help="the name of the metric that defines what the rule monitors.",
        )
        metric_trigger.metric_namespace = AAZStrArg(
            options=["metric-namespace"],
            help="the namespace of the metric that defines what the rule monitors.",
            nullable=True,
        )
        metric_trigger.metric_resource_location = AAZStrArg(
            options=["metric-resource-location"],
            help="the location of the resource the rule monitors.",
            nullable=True,
        )
        metric_trigger.metric_resource_uri = AAZStrArg(
            options=["metric-resource-uri"],
            help="the resource identifier of the resource the rule monitors.",
        )
        metric_trigger.operator = AAZStrArg(
            options=["operator"],
            help="the operator that is used to compare the metric data and the threshold.",
            enum={"Equals": "Equals", "GreaterThan": "GreaterThan", "GreaterThanOrEqual": "GreaterThanOrEqual", "LessThan": "LessThan", "LessThanOrEqual": "LessThanOrEqual", "NotEquals": "NotEquals"},
        )
        metric_trigger.statistic = AAZStrArg(
            options=["statistic"],
            help="the metric statistic type. How the metrics from multiple instances are combined.",
            enum={"Average": "Average", "Count": "Count", "Max": "Max", "Min": "Min", "Sum": "Sum"},
        )
        metric_trigger.threshold = AAZFloatArg(
            options=["threshold"],
            help="the threshold of the metric that triggers the scale action.",
        )
        metric_trigger.time_aggregation = AAZStrArg(
            options=["time-aggregation"],
            help="time aggregation type. How the data that is collected should be combined over time. The default value is Average.",
            enum={"Average": "Average", "Count": "Count", "Last": "Last", "Maximum": "Maximum", "Minimum": "Minimum", "Total": "Total"},
        )
        metric_trigger.time_grain = AAZDurationArg(
            options=["time-grain"],
            help="the granularity of metrics the rule monitors. Must be one of the predefined values returned from metric definitions for the metric. Must be between 12 hours and 1 minute.",
        )
        metric_trigger.time_window = AAZDurationArg(
            options=["time-window"],
            help="the range of time in which instance data is collected. This value must be greater than the delay in metric collection, which can vary from resource-to-resource. Must be between 12 hours and 5 minutes.",
        )

        dimensions = cls._args_schema.profiles.Element.rules.Element.metric_trigger.dimensions
        dimensions.Element = AAZObjectArg(
            nullable=True,
        )

        _element = cls._args_schema.profiles.Element.rules.Element.metric_trigger.dimensions.Element
        _element.dimension_name = AAZStrArg(
            options=["dimension-name"],
            help="Name of the dimension.",
        )
        _element.operator = AAZStrArg(
            options=["operator"],
            help="the dimension operator. Only 'Equals' and 'NotEquals' are supported. 'Equals' being equal to any of the values. 'NotEquals' being not equal to all of the values",
            enum={"Equals": "Equals", "NotEquals": "NotEquals"},
        )
        _element.values = AAZListArg(
            options=["values"],
            help="list of dimension values. For example: [\"App1\",\"App2\"].",
        )

        values = cls._args_schema.profiles.Element.rules.Element.metric_trigger.dimensions.Element.values
        values.Element = AAZStrArg(
            nullable=True,
        )

        scale_action = cls._args_schema.profiles.Element.rules.Element.scale_action
        scale_action.cooldown = AAZDurationArg(
            options=["cooldown"],
            help="the amount of time to wait since the last scaling action before this action occurs. It must be between 1 week and 1 minute in ISO 8601 format.",
        )
        scale_action.direction = AAZStrArg(
            options=["direction"],
            help="the scale direction. Whether the scaling action increases or decreases the number of instances.",
            enum={"Decrease": "Decrease", "Increase": "Increase", "None": "None"},
        )
        scale_action.type = AAZStrArg(
            options=["type"],
            help="the type of action that should occur when the scale rule fires.",
            enum={"ChangeCount": "ChangeCount", "ExactCount": "ExactCount", "PercentChangeCount": "PercentChangeCount", "ServiceAllowedNextValue": "ServiceAllowedNextValue"},
        )
        scale_action.value = AAZStrArg(
            options=["value"],
            help="the number of instances that are involved in the scaling action. This value must be 1 or greater. The default value is 1.",
            nullable=True,
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.AutoscaleSettingsGet(ctx=self.ctx)()
        self.pre_instance_update(self.ctx.vars.instance)
        self.InstanceUpdateByJson(ctx=self.ctx)()
        self.InstanceUpdateByGeneric(ctx=self.ctx)()
        self.post_instance_update(self.ctx.vars.instance)
        self.AutoscaleSettingsCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    @register_callback
    def pre_instance_update(self, instance):
        pass

    @register_callback
    def post_instance_update(self, instance):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class AutoscaleSettingsGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourcegroups/{resourceGroupName}/providers/Microsoft.Insights/autoscalesettings/{autoscaleSettingName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "autoscaleSettingName", self.ctx.args.autoscale_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2022-10-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _UpdateHelper._build_schema_autoscale_setting_resource_read(cls._schema_on_200)

            return cls._schema_on_200

    class AutoscaleSettingsCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200, 201]:
                return self.on_200_201(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourcegroups/{resourceGroupName}/providers/Microsoft.Insights/autoscalesettings/{autoscaleSettingName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "autoscaleSettingName", self.ctx.args.autoscale_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2022-10-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=self.ctx.vars.instance,
            )

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()
            _UpdateHelper._build_schema_autoscale_setting_resource_read(cls._schema_on_200_201)

            return cls._schema_on_200_201

    class InstanceUpdateByJson(AAZJsonInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance(self.ctx.vars.instance)

        def _update_instance(self, instance):
            _instance_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=instance,
                typ=AAZObjectType
            )
            _builder.set_prop("properties", AAZObjectType, ".", typ_kwargs={"flags": {"required": True, "client_flatten": True}})
            _builder.set_prop("tags", AAZDictType, ".tags")

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("enabled", AAZBoolType, ".enabled")
                properties.set_prop("name", AAZStrType, ".autoscale_name")
                properties.set_prop("notifications", AAZListType, ".notifications")
                properties.set_prop("predictiveAutoscalePolicy", AAZObjectType)
                properties.set_prop("profiles", AAZListType, ".profiles", typ_kwargs={"flags": {"required": True}})
                properties.set_prop("targetResourceLocation", AAZStrType, ".target_resource_location")
                properties.set_prop("targetResourceUri", AAZStrType, ".target_resource_uri")

            notifications = _builder.get(".properties.notifications")
            if notifications is not None:
                notifications.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.notifications[]")
            if _elements is not None:
                _elements.set_prop("email", AAZObjectType, ".email")
                _elements.set_prop("operation", AAZStrType, ".operation", typ_kwargs={"flags": {"required": True}})
                _elements.set_prop("webhooks", AAZListType, ".webhooks")

            email = _builder.get(".properties.notifications[].email")
            if email is not None:
                email.set_prop("customEmails", AAZListType, ".custom_emails")
                email.set_prop("sendToSubscriptionAdministrator", AAZBoolType, ".send_to_subscription_administrator")
                email.set_prop("sendToSubscriptionCoAdministrators", AAZBoolType, ".send_to_subscription_co_administrators")

            custom_emails = _builder.get(".properties.notifications[].email.customEmails")
            if custom_emails is not None:
                custom_emails.set_elements(AAZStrType, ".")

            webhooks = _builder.get(".properties.notifications[].webhooks")
            if webhooks is not None:
                webhooks.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.notifications[].webhooks[]")
            if _elements is not None:
                _elements.set_prop("properties", AAZDictType, ".properties")
                _elements.set_prop("serviceUri", AAZStrType, ".service_uri")

            properties = _builder.get(".properties.notifications[].webhooks[].properties")
            if properties is not None:
                properties.set_elements(AAZStrType, ".")

            predictive_autoscale_policy = _builder.get(".properties.predictiveAutoscalePolicy")
            if predictive_autoscale_policy is not None:
                predictive_autoscale_policy.set_prop("scaleLookAheadTime", AAZStrType, ".scale_look_ahead_time")
                predictive_autoscale_policy.set_prop("scaleMode", AAZStrType, ".scale_mode", typ_kwargs={"flags": {"required": True}})

            profiles = _builder.get(".properties.profiles")
            if profiles is not None:
                profiles.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.profiles[]")
            if _elements is not None:
                _elements.set_prop("capacity", AAZObjectType, ".capacity", typ_kwargs={"flags": {"required": True}})
                _elements.set_prop("fixedDate", AAZObjectType, ".fixed_date")
                _elements.set_prop("name", AAZStrType, ".name", typ_kwargs={"flags": {"required": True}})
                _elements.set_prop("recurrence", AAZObjectType, ".recurrence")
                _elements.set_prop("rules", AAZListType, ".rules", typ_kwargs={"flags": {"required": True}})

            capacity = _builder.get(".properties.profiles[].capacity")
            if capacity is not None:
                capacity.set_prop("default", AAZStrType, ".default", typ_kwargs={"flags": {"required": True}})
                capacity.set_prop("maximum", AAZStrType, ".maximum", typ_kwargs={"flags": {"required": True}})
                capacity.set_prop("minimum", AAZStrType, ".minimum", typ_kwargs={"flags": {"required": True}})

            fixed_date = _builder.get(".properties.profiles[].fixedDate")
            if fixed_date is not None:
                fixed_date.set_prop("end", AAZStrType, ".end", typ_kwargs={"flags": {"required": True}})
                fixed_date.set_prop("start", AAZStrType, ".start", typ_kwargs={"flags": {"required": True}})
                fixed_date.set_prop("timeZone", AAZStrType, ".time_zone")

            recurrence = _builder.get(".properties.profiles[].recurrence")
            if recurrence is not None:
                recurrence.set_prop("frequency", AAZStrType, ".frequency", typ_kwargs={"flags": {"required": True}})
                recurrence.set_prop("schedule", AAZObjectType, ".schedule", typ_kwargs={"flags": {"required": True}})

            schedule = _builder.get(".properties.profiles[].recurrence.schedule")
            if schedule is not None:
                schedule.set_prop("days", AAZListType, ".days", typ_kwargs={"flags": {"required": True}})
                schedule.set_prop("hours", AAZListType, ".hours", typ_kwargs={"flags": {"required": True}})
                schedule.set_prop("minutes", AAZListType, ".minutes", typ_kwargs={"flags": {"required": True}})
                schedule.set_prop("timeZone", AAZStrType, ".time_zone", typ_kwargs={"flags": {"required": True}})

            days = _builder.get(".properties.profiles[].recurrence.schedule.days")
            if days is not None:
                days.set_elements(AAZStrType, ".")

            hours = _builder.get(".properties.profiles[].recurrence.schedule.hours")
            if hours is not None:
                hours.set_elements(AAZIntType, ".")

            minutes = _builder.get(".properties.profiles[].recurrence.schedule.minutes")
            if minutes is not None:
                minutes.set_elements(AAZIntType, ".")

            rules = _builder.get(".properties.profiles[].rules")
            if rules is not None:
                rules.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.profiles[].rules[]")
            if _elements is not None:
                _elements.set_prop("metricTrigger", AAZObjectType, ".metric_trigger", typ_kwargs={"flags": {"required": True}})
                _elements.set_prop("scaleAction", AAZObjectType, ".scale_action", typ_kwargs={"flags": {"required": True}})

            metric_trigger = _builder.get(".properties.profiles[].rules[].metricTrigger")
            if metric_trigger is not None:
                metric_trigger.set_prop("dimensions", AAZListType, ".dimensions")
                metric_trigger.set_prop("dividePerInstance", AAZBoolType, ".divide_per_instance")
                metric_trigger.set_prop("metricName", AAZStrType, ".metric_name", typ_kwargs={"flags": {"required": True}})
                metric_trigger.set_prop("metricNamespace", AAZStrType, ".metric_namespace")
                metric_trigger.set_prop("metricResourceLocation", AAZStrType, ".metric_resource_location")
                metric_trigger.set_prop("metricResourceUri", AAZStrType, ".metric_resource_uri", typ_kwargs={"flags": {"required": True}})
                metric_trigger.set_prop("operator", AAZStrType, ".operator", typ_kwargs={"flags": {"required": True}})
                metric_trigger.set_prop("statistic", AAZStrType, ".statistic", typ_kwargs={"flags": {"required": True}})
                metric_trigger.set_prop("threshold", AAZFloatType, ".threshold", typ_kwargs={"flags": {"required": True}})
                metric_trigger.set_prop("timeAggregation", AAZStrType, ".time_aggregation", typ_kwargs={"flags": {"required": True}})
                metric_trigger.set_prop("timeGrain", AAZStrType, ".time_grain", typ_kwargs={"flags": {"required": True}})
                metric_trigger.set_prop("timeWindow", AAZStrType, ".time_window", typ_kwargs={"flags": {"required": True}})

            dimensions = _builder.get(".properties.profiles[].rules[].metricTrigger.dimensions")
            if dimensions is not None:
                dimensions.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.profiles[].rules[].metricTrigger.dimensions[]")
            if _elements is not None:
                _elements.set_prop("DimensionName", AAZStrType, ".dimension_name", typ_kwargs={"flags": {"required": True}})
                _elements.set_prop("Operator", AAZStrType, ".operator", typ_kwargs={"flags": {"required": True}})
                _elements.set_prop("Values", AAZListType, ".values", typ_kwargs={"flags": {"required": True}})

            values = _builder.get(".properties.profiles[].rules[].metricTrigger.dimensions[].Values")
            if values is not None:
                values.set_elements(AAZStrType, ".")

            scale_action = _builder.get(".properties.profiles[].rules[].scaleAction")
            if scale_action is not None:
                scale_action.set_prop("cooldown", AAZStrType, ".cooldown", typ_kwargs={"flags": {"required": True}})
                scale_action.set_prop("direction", AAZStrType, ".direction", typ_kwargs={"flags": {"required": True}})
                scale_action.set_prop("type", AAZStrType, ".type", typ_kwargs={"flags": {"required": True}})
                scale_action.set_prop("value", AAZStrType, ".value")

            tags = _builder.get(".tags")
            if tags is not None:
                tags.set_elements(AAZStrType, ".")

            return _instance_value

    class InstanceUpdateByGeneric(AAZGenericInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance_by_generic(
                self.ctx.vars.instance,
                self.ctx.generic_update_args
            )


class _UpdateHelper:
    """Helper class for Update"""

    _schema_autoscale_setting_resource_read = None

    @classmethod
    def _build_schema_autoscale_setting_resource_read(cls, _schema):
        if cls._schema_autoscale_setting_resource_read is not None:
            _schema.id = cls._schema_autoscale_setting_resource_read.id
            _schema.location = cls._schema_autoscale_setting_resource_read.location
            _schema.name = cls._schema_autoscale_setting_resource_read.name
            _schema.properties = cls._schema_autoscale_setting_resource_read.properties
            _schema.system_data = cls._schema_autoscale_setting_resource_read.system_data
            _schema.tags = cls._schema_autoscale_setting_resource_read.tags
            _schema.type = cls._schema_autoscale_setting_resource_read.type
            return

        cls._schema_autoscale_setting_resource_read = _schema_autoscale_setting_resource_read = AAZObjectType()

        autoscale_setting_resource_read = _schema_autoscale_setting_resource_read
        autoscale_setting_resource_read.id = AAZStrType(
            flags={"read_only": True},
        )
        autoscale_setting_resource_read.location = AAZStrType(
            flags={"required": True},
        )
        autoscale_setting_resource_read.name = AAZStrType(
            flags={"read_only": True},
        )
        autoscale_setting_resource_read.properties = AAZObjectType(
            flags={"required": True, "client_flatten": True},
        )
        autoscale_setting_resource_read.system_data = AAZObjectType(
            serialized_name="systemData",
            flags={"read_only": True},
        )
        cls._build_schema_system_data_read(autoscale_setting_resource_read.system_data)
        autoscale_setting_resource_read.tags = AAZDictType()
        autoscale_setting_resource_read.type = AAZStrType(
            flags={"read_only": True},
        )

        properties = _schema_autoscale_setting_resource_read.properties
        properties.enabled = AAZBoolType()
        properties.name = AAZStrType()
        properties.notifications = AAZListType()
        properties.predictive_autoscale_policy = AAZObjectType(
            serialized_name="predictiveAutoscalePolicy",
        )
        properties.profiles = AAZListType(
            flags={"required": True},
        )
        properties.target_resource_location = AAZStrType(
            serialized_name="targetResourceLocation",
        )
        properties.target_resource_uri = AAZStrType(
            serialized_name="targetResourceUri",
        )

        notifications = _schema_autoscale_setting_resource_read.properties.notifications
        notifications.Element = AAZObjectType()

        _element = _schema_autoscale_setting_resource_read.properties.notifications.Element
        _element.email = AAZObjectType()
        _element.operation = AAZStrType(
            flags={"required": True},
        )
        _element.webhooks = AAZListType()

        email = _schema_autoscale_setting_resource_read.properties.notifications.Element.email
        email.custom_emails = AAZListType(
            serialized_name="customEmails",
        )
        email.send_to_subscription_administrator = AAZBoolType(
            serialized_name="sendToSubscriptionAdministrator",
        )
        email.send_to_subscription_co_administrators = AAZBoolType(
            serialized_name="sendToSubscriptionCoAdministrators",
        )

        custom_emails = _schema_autoscale_setting_resource_read.properties.notifications.Element.email.custom_emails
        custom_emails.Element = AAZStrType()

        webhooks = _schema_autoscale_setting_resource_read.properties.notifications.Element.webhooks
        webhooks.Element = AAZObjectType()

        _element = _schema_autoscale_setting_resource_read.properties.notifications.Element.webhooks.Element
        _element.properties = AAZDictType()
        _element.service_uri = AAZStrType(
            serialized_name="serviceUri",
        )

        properties = _schema_autoscale_setting_resource_read.properties.notifications.Element.webhooks.Element.properties
        properties.Element = AAZStrType()

        predictive_autoscale_policy = _schema_autoscale_setting_resource_read.properties.predictive_autoscale_policy
        predictive_autoscale_policy.scale_look_ahead_time = AAZStrType(
            serialized_name="scaleLookAheadTime",
        )
        predictive_autoscale_policy.scale_mode = AAZStrType(
            serialized_name="scaleMode",
            flags={"required": True},
        )

        profiles = _schema_autoscale_setting_resource_read.properties.profiles
        profiles.Element = AAZObjectType()

        _element = _schema_autoscale_setting_resource_read.properties.profiles.Element
        _element.capacity = AAZObjectType(
            flags={"required": True},
        )
        _element.fixed_date = AAZObjectType(
            serialized_name="fixedDate",
        )
        _element.name = AAZStrType(
            flags={"required": True},
        )
        _element.recurrence = AAZObjectType()
        _element.rules = AAZListType(
            flags={"required": True},
        )

        capacity = _schema_autoscale_setting_resource_read.properties.profiles.Element.capacity
        capacity.default = AAZStrType(
            flags={"required": True},
        )
        capacity.maximum = AAZStrType(
            flags={"required": True},
        )
        capacity.minimum = AAZStrType(
            flags={"required": True},
        )

        fixed_date = _schema_autoscale_setting_resource_read.properties.profiles.Element.fixed_date
        fixed_date.end = AAZStrType(
            flags={"required": True},
        )
        fixed_date.start = AAZStrType(
            flags={"required": True},
        )
        fixed_date.time_zone = AAZStrType(
            serialized_name="timeZone",
        )

        recurrence = _schema_autoscale_setting_resource_read.properties.profiles.Element.recurrence
        recurrence.frequency = AAZStrType(
            flags={"required": True},
        )
        recurrence.schedule = AAZObjectType(
            flags={"required": True},
        )

        schedule = _schema_autoscale_setting_resource_read.properties.profiles.Element.recurrence.schedule
        schedule.days = AAZListType(
            flags={"required": True},
        )
        schedule.hours = AAZListType(
            flags={"required": True},
        )
        schedule.minutes = AAZListType(
            flags={"required": True},
        )
        schedule.time_zone = AAZStrType(
            serialized_name="timeZone",
            flags={"required": True},
        )

        days = _schema_autoscale_setting_resource_read.properties.profiles.Element.recurrence.schedule.days
        days.Element = AAZStrType()

        hours = _schema_autoscale_setting_resource_read.properties.profiles.Element.recurrence.schedule.hours
        hours.Element = AAZIntType()

        minutes = _schema_autoscale_setting_resource_read.properties.profiles.Element.recurrence.schedule.minutes
        minutes.Element = AAZIntType()

        rules = _schema_autoscale_setting_resource_read.properties.profiles.Element.rules
        rules.Element = AAZObjectType()

        _element = _schema_autoscale_setting_resource_read.properties.profiles.Element.rules.Element
        _element.metric_trigger = AAZObjectType(
            serialized_name="metricTrigger",
            flags={"required": True},
        )
        _element.scale_action = AAZObjectType(
            serialized_name="scaleAction",
            flags={"required": True},
        )

        metric_trigger = _schema_autoscale_setting_resource_read.properties.profiles.Element.rules.Element.metric_trigger
        metric_trigger.dimensions = AAZListType()
        metric_trigger.divide_per_instance = AAZBoolType(
            serialized_name="dividePerInstance",
        )
        metric_trigger.metric_name = AAZStrType(
            serialized_name="metricName",
            flags={"required": True},
        )
        metric_trigger.metric_namespace = AAZStrType(
            serialized_name="metricNamespace",
        )
        metric_trigger.metric_resource_location = AAZStrType(
            serialized_name="metricResourceLocation",
        )
        metric_trigger.metric_resource_uri = AAZStrType(
            serialized_name="metricResourceUri",
            flags={"required": True},
        )
        metric_trigger.operator = AAZStrType(
            flags={"required": True},
        )
        metric_trigger.statistic = AAZStrType(
            flags={"required": True},
        )
        metric_trigger.threshold = AAZFloatType(
            flags={"required": True},
        )
        metric_trigger.time_aggregation = AAZStrType(
            serialized_name="timeAggregation",
            flags={"required": True},
        )
        metric_trigger.time_grain = AAZStrType(
            serialized_name="timeGrain",
            flags={"required": True},
        )
        metric_trigger.time_window = AAZStrType(
            serialized_name="timeWindow",
            flags={"required": True},
        )

        dimensions = _schema_autoscale_setting_resource_read.properties.profiles.Element.rules.Element.metric_trigger.dimensions
        dimensions.Element = AAZObjectType()

        _element = _schema_autoscale_setting_resource_read.properties.profiles.Element.rules.Element.metric_trigger.dimensions.Element
        _element.dimension_name = AAZStrType(
            serialized_name="DimensionName",
            flags={"required": True},
        )
        _element.operator = AAZStrType(
            serialized_name="Operator",
            flags={"required": True},
        )
        _element.values = AAZListType(
            serialized_name="Values",
            flags={"required": True},
        )

        values = _schema_autoscale_setting_resource_read.properties.profiles.Element.rules.Element.metric_trigger.dimensions.Element.values
        values.Element = AAZStrType()

        scale_action = _schema_autoscale_setting_resource_read.properties.profiles.Element.rules.Element.scale_action
        scale_action.cooldown = AAZStrType(
            flags={"required": True},
        )
        scale_action.direction = AAZStrType(
            flags={"required": True},
        )
        scale_action.type = AAZStrType(
            flags={"required": True},
        )
        scale_action.value = AAZStrType()

        tags = _schema_autoscale_setting_resource_read.tags
        tags.Element = AAZStrType()

        _schema.id = cls._schema_autoscale_setting_resource_read.id
        _schema.location = cls._schema_autoscale_setting_resource_read.location
        _schema.name = cls._schema_autoscale_setting_resource_read.name
        _schema.properties = cls._schema_autoscale_setting_resource_read.properties
        _schema.system_data = cls._schema_autoscale_setting_resource_read.system_data
        _schema.tags = cls._schema_autoscale_setting_resource_read.tags
        _schema.type = cls._schema_autoscale_setting_resource_read.type

    _schema_system_data_read = None

    @classmethod
    def _build_schema_system_data_read(cls, _schema):
        if cls._schema_system_data_read is not None:
            _schema.created_at = cls._schema_system_data_read.created_at
            _schema.created_by = cls._schema_system_data_read.created_by
            _schema.created_by_type = cls._schema_system_data_read.created_by_type
            _schema.last_modified_at = cls._schema_system_data_read.last_modified_at
            _schema.last_modified_by = cls._schema_system_data_read.last_modified_by
            _schema.last_modified_by_type = cls._schema_system_data_read.last_modified_by_type
            return

        cls._schema_system_data_read = _schema_system_data_read = AAZObjectType(
            flags={"read_only": True}
        )

        system_data_read = _schema_system_data_read
        system_data_read.created_at = AAZStrType(
            serialized_name="createdAt",
        )
        system_data_read.created_by = AAZStrType(
            serialized_name="createdBy",
        )
        system_data_read.created_by_type = AAZStrType(
            serialized_name="createdByType",
        )
        system_data_read.last_modified_at = AAZStrType(
            serialized_name="lastModifiedAt",
        )
        system_data_read.last_modified_by = AAZStrType(
            serialized_name="lastModifiedBy",
        )
        system_data_read.last_modified_by_type = AAZStrType(
            serialized_name="lastModifiedByType",
        )

        _schema.created_at = cls._schema_system_data_read.created_at
        _schema.created_by = cls._schema_system_data_read.created_by
        _schema.created_by_type = cls._schema_system_data_read.created_by_type
        _schema.last_modified_at = cls._schema_system_data_read.last_modified_at
        _schema.last_modified_by = cls._schema_system_data_read.last_modified_by
        _schema.last_modified_by_type = cls._schema_system_data_read.last_modified_by_type


__all__ = ["Update"]
